//
// Copyright (C) 2006 Institut fuer Telematik, Universitaet Karlsruhe (TH)
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//

/**
* @file MovementGenerator.cc
* @author Helge Backhaus
*/

#include "MovementGenerator.h"

std::ostream& operator<<(std::ostream& Stream, const NeighborMapEntry& e)
{
    return Stream << "Positon: " << e.position << " Direction: " << e.direction;
}

MovementGenerator::MovementGenerator(double areaDimension, double speed, NeighborMap *Neighbors)
{
    this->areaDimension = areaDimension;
    this->speed = speed;
    this->Neighbors = Neighbors;

    Vector2D center(areaDimension / 2, areaDimension / 2);
    position.x = uniform(0.0, areaDimension);
    position.y = uniform(0.0, areaDimension);
    direction = center - position;
    direction.normalize();
}

Vector2D MovementGenerator::getPosition()
{
    return position;
}

void MovementGenerator::testBounds()
{
    if(position.x < 0.0) position.x = 0.0;
    if(position.x > areaDimension) position.x = areaDimension;
    if(position.y < 0.0) position.y = 0.0;
    if(position.y > areaDimension) position.y = areaDimension;
}

void MovementGenerator::flock()
{
    Vector2D separation, alignment, cohesion, toTarget;
    int NeighborCount = 0;

    for(itNeighbors = Neighbors->begin(); itNeighbors != Neighbors->end(); ++itNeighbors)
        if(position.distanceSqr(itNeighbors->second.position) < 5.0 && direction.cosAngle(itNeighbors->second.position - position) > -0.75) {
            separation += position - itNeighbors->second.position;
            alignment += itNeighbors->second.direction;
            cohesion += itNeighbors->second.position;
            ++NeighborCount;
        }

    if(NeighborCount > 0) {
        cohesion /= (double)NeighborCount;
        cohesion = cohesion - position;
        separation.normalize();
        alignment.normalize();
        cohesion.normalize();
    }
    toTarget = target - position;
    toTarget.normalize();

    direction = separation * 0.4 + alignment * 0.1 + cohesion * 0.35 + toTarget * 0.25;
    direction.normalize();
}
