//
// Copyright (C) 2008 Institut fuer Telematik, Universitaet Karlsruhe (TH)
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//

/**
 * @file BootstrapList.cc
 * @author Ingmar Baumgart
 */


#include "BootstrapList.h"

Define_Module(BootstrapList);

BootstrapList::BootstrapList()
{
    timerMsg = NULL;
}

BootstrapList::~BootstrapList()
{
    cancelAndDelete(timerMsg);
}

int BootstrapList::numInitStages() const
{
    return MAX_STAGE_OVERLAY + 1;
}

void BootstrapList::initialize(int stage) 
{
    if (stage != MAX_STAGE_OVERLAY) {
        return;
    }
    
    overlay = NULL;
        
    timerMsg = new cMessage("BootstrapListTimer");
//    scheduleAt(simTime() + timerInterval, timerMsg);
}

void BootstrapList::handleMessage(cMessage *msg)
{
    if (msg == timerMsg) {
        handleBootstrapListTimerExpired();
        scheduleAt(simTime() + timerInterval, msg);
    } else {
        throw new cException("BootstrapList::handleMessage(): "
                             "Received unknown message type!");
    }
}


void BootstrapList::handleBootstrapListTimerExpired()
{

}

void BootstrapList::insertBootstrapNode(const NodeHandle& node)
{
    Enter_Method_Silent();
    
    EV << "New bootstrap node " << node << endl;
}

void BootstrapList::finish()
{
}


