//
// Copyright (C) 2006 Institut fuer Telematik, Universitaet Karlsruhe (TH)
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//

/**
 * @file PingCache.cc
 * @author Antonio Zea
 */


#include "PingCache.h"


std::ostream& operator<<(std::ostream& os, const PingCache::PingCacheEntry &p) 
{
    os << p.rtt << " (i=" << p.insertTime << ")";
    return os;
}


Define_Module(PingCache);

void PingCache::initialize() 
{
    WATCH_MAP(pingCache);
}

void PingCache::setAddressPinged(const TransportAddress& handle)
{
    PingCacheEntry entry;
    
    entry.insertTime = simulation.simTime();
    entry.rtt = PINGSTATE_WAITING;
    entry.nodeRef = NodeHandle::UNSPECIFIED_NODE;
    
    pingCache[handle] = entry;
}

void PingCache::addressTimeout(const TransportAddress& handle)
{
    PingCacheEntry entry;
    
    entry.insertTime = simulation.simTime();
    entry.rtt = PINGSTATE_TIMEOUT;
    entry.nodeRef = NodeHandle::UNSPECIFIED_NODE;
    
    pingCache[handle] = entry;
}


void PingCache::insertAddressRtt(const NodeHandle &add, simtime_t rtt) 
{
    PingCacheEntry entry;
    
    entry.insertTime = simulation.simTime();
    entry.rtt = rtt;
    entry.nodeRef = add;
    
    TransportAddress ta = add;
    pingCache[ta] = entry;
}

simtime_t PingCache::getAddressRtt(const TransportAddress &add) 
{
    if (par("disablePingCache")) return PINGSTATE_UNKNOWN;  /* cache disabled */
    
    if (pingCache.count(add) == 0) return PINGSTATE_UNKNOWN; /* entry not there */
    
    PingCacheEntry entry = pingCache[add];
    
    if (simulation.simTime() - entry.rtt >= (simtime_t)par("pingExpirationTime")) 
        return PINGSTATE_UNKNOWN;   /* entry expired */
    else 
        return entry.rtt;
}

NodeHandle PingCache::getNodeHandle(const TransportAddress &add) 
{
    if (pingCache.count(add) == 0) opp_error("PingCache.cc: getNodeHandle was asked for a non-existant node reference.");
    return pingCache[add].nodeRef;
}
