//
// Copyright (C) 2006 Institut fuer Telematik, Universitaet Karlsruhe (TH)
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//

/**
 * @file PingCache.h
 * @author Antonio Zea
 */


#ifndef __PINGCACHE_H_
#define __PINGCACHE_H_

#include <omnetpp.h>

#include <NodeHandle.h>
#include <OverlayKey.h>
#include <map>


enum {
    PINGSTATE_UNKNOWN = -1,
    PINGSTATE_TIMEOUT = -2,
    PINGSTATE_WAITING = -3
};
    

class PingCache : public cSimpleModule
{
    protected:
        struct PingCacheEntry {
            simtime_t  insertTime;
            simtime_t  rtt;
            NodeHandle nodeRef;
        };
        
        std::map<TransportAddress, PingCacheEntry> pingCache;
    
        void initialize();

    public:        
        void setAddressPinged(const TransportAddress& handle);
        void insertAddressRtt(const NodeHandle &add, simtime_t time);
        void addressTimeout(const TransportAddress& handle);
        simtime_t getAddressRtt(const TransportAddress &add);
        NodeHandle getNodeHandle(const TransportAddress &add);

        friend std::ostream& operator<<(std::ostream& os, const PingCacheEntry &);
};


#endif
