//
// Copyright (C) 2008 Institut fuer Telematik, Universitaet Karlsruhe (TH)
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//

/**
 * @file RecursiveLookup.cc
 * @author Bernhard Heep
 */

#include <CommonMessages_m.h>
#include <RecursiveLookup.h>

RecursiveLookup::RecursiveLookup(BaseOverlay* overlay) :
    overlay(overlay)
{
    valid = false;
}

RecursiveLookup::~RecursiveLookup()
{
    overlay->removeLookup(this);
}

void RecursiveLookup::lookup(const OverlayKey& key, int numSiblings,
                             int hopCountMax, int retries,
                             LookupListener* listener)
{
    this->listener = listener;

    FindNodeCall* call = new FindNodeCall("FindNodeCall");
    call->setLookupKey(key);
    call->setNumRedundantNodes(0);
    call->setNumSiblings(numSiblings);
    call->setLength(FINDNODECALL_L(call));

    nonce = overlay->sendRouteRpcCall(OVERLAY_COMP, key, call,
                                      -1, retries, -1, this);
}

const NodeVector& RecursiveLookup::getResult() const
{
    return siblings;
}

bool RecursiveLookup::isValid() const
{
    return valid;
}

void RecursiveLookup::abortLookup()
{
    overlay->cancelRpcMessage(nonce);

    if (listener != NULL) {
        delete listener;
        listener = NULL;
    }

    delete this;
}

uint RecursiveLookup::getAccumulatedHops() const
{
    return 0; //TODO
}

void RecursiveLookup::handleRpcTimeout(BaseCallMessage* msg,
                                       const TransportAddress& dest, int rpcId,
                                       const OverlayKey& destKey)
{
    valid = false;
}

void RecursiveLookup::handleRpcResponse(BaseResponseMessage* msg, int rpcId,
                                        simtime_t rtt)
{
    FindNodeResponse* findNodeResponse = check_and_cast<FindNodeResponse*>(msg);

    if (findNodeResponse->getSiblings() &&
        findNodeResponse->getClosestNodesArraySize() > 0) {
        valid = true;
        for (uint i = 0; i < findNodeResponse->getClosestNodesArraySize();
             i++) {
            siblings.push_back(findNodeResponse->getClosestNodes(i));
        }
    }

    // inform listener
    if (listener != NULL) {
        listener->lookupFinished(this);
        listener = NULL;
    }
}
