//
// Copyright (C) 2006 Institut fuer Telematik, Universitaet Karlsruhe (TH)
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//

/**
 * @file ConnectivityProbe.h
 * @author Helge Backhaus
 */

#ifndef __CONNECTIVITYPROBE_H__
#define __CONNECTIVITYPROBE_H__

#include <omnetpp.h>
#include <NodeHandle.h>
#include <VastDefs.h>
#include <Vast.h>

typedef std::set<NodeHandle> NodeSet;

class VTopologyNode
{
    public:
        VTopologyNode(int moduleID);
        Vast* module() const;

        int componentCounter;
        int moduleID;
};

typedef std::map<OverlayKey, VTopologyNode> VTopology;

class ConnectivityProbe : public cSimpleModule
{
    public:
        virtual void initialize();
        virtual void handleMessage(cMessage* msg);
        virtual ~ConnectivityProbe();

    private:
        void extractTopology();
        int findComponent(OverlayKey key, int Component);

        double probeIntervall;
        cMessage* probeTimer;
        VTopology Topology;

        // statistics
        cOutVector cOV_NodeCount;
        cOutVector cOV_MaximumComponent;
        cOutVector cOV_ZeroMissingNeighbors;
        cOutVector cOV_OneMissingNeighbor;
        cOutVector cOV_TwoMissingNeighbor;
        cOutVector cOV_ThreeMissingNeighbor;
        cOutVector cOV_FourMissingNeighbor;
        cOutVector cOV_FiveMissingNeighbors;
        cOutVector cOV_SeveralMissingNeighbors;
        cOutVector cOV_AverageMissingNeighbors;
        cOutVector cOV_MaxMissingNeighbors;
};

#endif



