//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//


#include "BackupNodeData.h"

BackupNodeData::BackupNodeData(SubSpace subSpace) {
	this->subSpace = subSpace;
	this->backupDelay = 1;
	this->size=0;
}

int BackupNodeData::getId() {
	return subSpace.getId();
}
int BackupNodeData::getSize() {
	return size;
}

void BackupNodeData::setTree(std::list<TreeNode> tree) {
	this->tree = tree;
	size=0;
	std::list<TreeNode>::iterator t;
	for(t = tree.begin(); t != tree.end(); ++t) {
		if( t->playerId > 0) {
			++size;
		}
	}
}
std::list<TreeNode> BackupNodeData::getTree() {
	return this->tree;
}
SubSpace BackupNodeData::getSubSpace() {
	return this->subSpace;
}
void BackupNodeData::setSubSpace(SubSpace subSpace) {
	this->subSpace = subSpace;
}

void BackupNodeData::addDelay(double time, IPAddress ip) {
	Delay delay;
	delay.delay = time;
	delay.ip = ip;
	delays.push_back(delay);
}
Delay BackupNodeData::getMaxDelay() {
	if ( delays.size() > 0 ) {
		delays.sort();
		Delay returnDelay = delays.back();
		lastMaxDelay = returnDelay;
		delays.clear();
		return returnDelay;
	}
	else {
		Delay dummy;
		dummy.delay = 0; //will never replaced
		dummy.ip = (IPAddress) "127.0.0.1";
		return dummy;
	}
}
double BackupNodeData::getBackupDelay() {
	return backupDelay;
}
void BackupNodeData::setBackupDelay(double delay) {
	lastBackupDelays.push_back( delay);
	if(lastBackupDelays.size() > 5 ) lastBackupDelays.pop_front();
	double avg=0;
	for(lB = lastBackupDelays.begin(); lB != lastBackupDelays.end(); ++lB) {
		avg += *lB;
	}
	backupDelay = avg/lastBackupDelays.size();
}

std::ostream& operator<<(std::ostream& Stream, BackupNodeData b)
{
    return Stream << "SubSpaceId: " << b.getSubSpace().getId() << " PlayerListSize: " << b.tree.size() << " last received Message: " << b.lastMessage << " own Delay: " << b.backupDelay << " Last Max Delay: " << b.lastMaxDelay.delay << " " << b.lastMaxDelay.ip << " \n";
}

bool BackupNodeData::operator<(const BackupNodeData backupNode) const {
	if ( lastMessage < backupNode.lastMessage ) return true;
	return false;
}

bool Delay::operator<(const Delay d) const {
	if ( delay < d.delay ) return true;
	return false;
}

