//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//


#include <CommonMessages_m.h>
#include <UDPControlInfo_m.h>
#include <IPAddressResolver.h>
#include <IPvXAddress.h>
#include <InterfaceTable.h>
#include <IPv4InterfaceData.h>
#include "SubSpace.h"
#include "LobbyServer.h"
#include "YMMOGMessage_m.h"


Define_Module(LobbyServer);
LobbyServer::~LobbyServer() {
//	playerOnlineTable->finish();
//	subSpaceTable->finish();
//	this->finish();
	// clean up
	//delete stabilizeTimer;
}

void LobbyServer::initialize()
{
	// Get Parameters
	gameSpaceSize = par("gameSpaceSize");
	subSpaceSize = par("subSpaceSize");
	// Initialisierung
	playerOnlineTable = check_and_cast<PlayerTable*>(parentModule()->submodule("playerOnlineTable"));
	subSpaceTable = check_and_cast<SubSpaceTable*>(simulation.moduleByPath("globalObserver.globalFunctions[0]"));
    // find a new bootstrap node and enroll to the bootstrap list
	bootstrapOracle->registerPeer(thisNode);

	subSpaceTable->createTable(gameSpaceSize);

	resendDelay = 1.5;

	// Initiate Timer
	pingTimer = new cMessage("pingTimer");
	pingDelay = 30;
	scheduleAt(simTime() + pingDelay, pingTimer);

	maxFailed = 3;

	maxFailedPings = 2;
	droppedMessages = 0;
	WATCH( droppedMessages );

	thereIsANotReadyCreate = false;
	// Statistics
	WATCH(completedLogins);
	completedLogins=0;
}
void LobbyServer::finish()
{
	// delete from bootstrap
	bootstrapOracle->removePeer(thisNode);
	finishStats();
}
void LobbyServer::processSelfTimers(cMessage* msg) {
	if(msg->isName("resendTimer")) {
		processResendTimer();
	}
	else if ( msg->isName("pingTimer") ) {
		processPingTimer() ;
	}
	else if (msg->isName("bandwithTimer") ) {
		processBandwithTimer();
	}
	else if ( msg->isName("stabilizeTimer") ) {
		processStabilizeTimer();
	}
}


void LobbyServer::processPingTimer() {
	selfMessages++;
	cancelEvent(pingTimer);
	std::list<PingMessage*>::iterator p= pings.begin();
	double next=simTime();
	while( p != pings.end() ) {
		if( ( (*p)->getTimeStamp() + pingDelay) < simTime() ) {
			(*p)->setTimeStamp(simTime());
			(*p)->setFailed( (*p)->getFailed() + 1 );
			if( (*p)->getFailed() > maxFailedPings ) {
				playerOnlineTable->remove((*p)->getDestip());
				delete *p;
				p = pings.erase(p);
			}
			else {
				if( next > (*p)->getTimeStamp() ) next = (*p)->getTimeStamp();
				sendPing( (*p) );
				++p;
			}
		}
		else { ++p; }
	}
	std::list<PlayerNode>::iterator pn;
	for(pn=playerOnlineTable->playerTable.begin(); pn != playerOnlineTable->playerTable.end(); ++pn) {
		if( !findOpenPing( pn->getIp() ) ) {
				PingMessage* pingMsg = new PingMessage("LobbyPing");
				pingMsg->setCommand(PING);
				pingMsg->setServiceType(REQUEST);
				pingMsg->setDestip( pn->getIp() );
				pingMsg->setSrcip(thisNode.ip.get4() );
				pingMsg->setFailed(0);
				pingMsg->setTimeStamp(simTime());
				pings.push_back(pingMsg);
				sendPing( pingMsg );
		}
	}
	scheduleAt(next + pingDelay, pingTimer);
}

void LobbyServer::sendPing(PingMessage* openPing) {
	PingMessage* pingMsg = new PingMessage("LOBBYPING");
	pingMsg->setCommand(PING);
	pingMsg->setServiceType(REQUEST);
	pingMsg->setSent(simTime());
	sendMsg(pingMsg, openPing->getDestip());
}

void LobbyServer::processPing(PingMessage* pingMsg) {
/**
 We use only Request/confirmation
**/
	deleteFromOpenPings(pingMsg->getSrcip());
	delete pingMsg;
}

int LobbyServer::login(int id, IPAddress ip, int Bandwith, int Cpu) {
	return playerOnlineTable->add(id,ip,Bandwith,Cpu);
}

bool LobbyServer::logout(int id) {
	return playerOnlineTable->remove(id);
}

bool LobbyServer::createResponsibleNode(int subSpaceId) {
	std::list<YMMOGMessage*>::iterator w = notReadyMessages.begin();
	//check if there is a CreateMsg already waiting
	while ( w != notReadyMessages.end() ) {
		if( (*w)->getCommand() == CREATE ) {
			CreateNodeMessage* cmsg = dynamic_cast<CreateNodeMessage*>(*w);
			if( cmsg != NULL && cmsg->getSubSpaceToCreate().getId() == subSpaceId && cmsg->getNodeType() == RESPONSIBLENODE ) {
				w = notReadyMessages.erase(w);
			}
			else { ++w; }
		}
		else {
			++w;
		}
	}
	//check if there is already an create on its way:
	w = dupedMessages.begin();
	while( w != dupedMessages.end() ) {
		if( (*w)->getCommand() == CREATE) {
			CreateNodeMessage* cmsg = dynamic_cast<CreateNodeMessage*>(*w);
			if( cmsg != NULL && cmsg->getSubSpaceToCreate().getId() == subSpaceId && cmsg->getNodeType() == RESPONSIBLENODE && cmsg->getJobType() == CREATENODE ) {
				return false;
			}
		}
		++w;
	}
	//Create a new CreateNodeMessage
	IPAddress destip = (playerOnlineTable->getHeadAndModify(subSpaceId, (IPAddress) "127.0.0.1", -110)).getIp();
	CreateNodeMessage* createMsg = new CreateNodeMessage();
	createMsg->setCommand(CREATE);
	createMsg->setServiceType(REQUEST);
	createMsg->setNodeType(RESPONSIBLENODE);
	createMsg->setJobType(CREATENODE);
	createMsg->setSubSpaceToCreate( *(subSpaceTable->getSubSpace(subSpaceId)) );
	if( destip != (IPAddress) "127.0.0.1") {
		sendMsg(createMsg, destip);
		return true;
	}
	else {
		notReadyMessages.push_back( createMsg );
		return false;
	}
}

void LobbyServer::informNeighbors(SubSpace *subSpace) {
	for( int i=0; i < 8; i++) {
		if( subSpace->getNeighbor(i) != (IPAddress) "127.0.0.1" ) {
			UpdateRespIpMessage* updateMsg = new UpdateRespIpMessage("Update Responsible Ip");
			updateMsg->setCommand(UPDATERESPIP);
			updateMsg->setServiceType(REQUEST);
			if( i < 3 ) { updateMsg->setSubSpaceId(subSpace->getId() - gameSpaceSize + i - 1 ); }
			else if( i == 3) { updateMsg->setSubSpaceId(subSpace->getId() - 1 ); }
			else if( i == 4) { updateMsg->setSubSpaceId(subSpace->getId() + 1 ); }
			else if (i >= 5) { updateMsg->setSubSpaceId(subSpace->getId() + gameSpaceSize + i - 6 ); }
			updateMsg->setDirection(7-i);
			updateMsg->setUpdatedIp(subSpace->getRespIp());
			sendMsg(updateMsg, subSpace->getNeighbor(i));
		}
	}
}


void LobbyServer::addSubSpaceToOpenLogins(int subSpaceId) {
	std::list<YMMOGMessage*>::iterator i = notReadyMessages.begin();
	while( i != notReadyMessages.end() ) {
		if( (*i)->getCommand() == LOGIN) {
			LoginMessage* loginMsg = dynamic_cast<LoginMessage*>(*i);
			if( loginMsg != NULL) {
				bool notComplete = false;
				for(uint k = 0; k != loginMsg->getResponsibleNodesArraySize(); ++k) {
					if( loginMsg->getResponsibleNodes(k).getId() == subSpaceId ) {
						loginMsg->setResponsibleNodes(k, (*subSpaceTable->getSubSpace( subSpaceId)) );
					}
					if( loginMsg->getResponsibleNodes(k).getRespIp() == (IPAddress) "127.0.0.1" )
						notComplete = true;
				}
				if ( !notComplete ) {
					sendMsg( loginMsg, loginMsg->getSrcip() );
					i = notReadyMessages.erase(i);
				}
				else {
					++i;
				}
			}
			else {
				++i;
			}
		} else { ++i; }
	}
}




void LobbyServer::sendWaitingRequests(int subSpaceId, int nodeType) {
	std::list<YMMOGMessage*>::iterator i = notReadyMessages.begin();
	while( i != notReadyMessages.end() ) {
		if( (*i)->getCommand() == REQUESTIP) {
			RequestIpMessage* requestMsg = dynamic_cast<RequestIpMessage*>(*i);
			if( requestMsg != NULL && requestMsg->getSubSpaceId() == subSpaceId && requestMsg->getNodeType() == nodeType) {
				if( nodeType == RESPONSIBLENODE ) {
					requestMsg->setRequestedIp( subSpaceTable->getSubSpace( subSpaceId )->getRespIp() );
				}
				else if ( nodeType == BACKUPNODE ) {
					requestMsg->setRequestedIp( subSpaceTable->getSubSpace( subSpaceId )->getBackupIp() );
				}
				sendMsg(requestMsg, requestMsg->getSrcip() );
				i = notReadyMessages.erase(i);
			} else {
				++i;
			}
		}
		else { ++i; }
	}
}

void LobbyServer::processLogin(LoginMessage* loginMsg) {
/**
	function is called everytime a login message arrives.
**/
	if( loginMsg->getServiceType() == REQUEST )
	{
		if( findOpenLogin(loginMsg->getSrcip() ) ) {
			// We already got the login. Its not ready or its already on its way. Timers will do the rest
			delete loginMsg;
			return;
		}
		// Get our Id and Position and add ourself to playertable
		int id = login( loginMsg->getId(), loginMsg->getSrcip(), loginMsg->getBandwith(), loginMsg->getCpu());
		// Check if we have some waiting Creates
		std::list<YMMOGMessage*>::iterator w = notReadyMessages.begin();
		while( w != notReadyMessages.end() ) {
			if( (*w)->getCommand() == CREATE ) {
				CreateNodeMessage* cmsg = dynamic_cast<CreateNodeMessage*>(*w);
				if( cmsg != NULL ) {
					IPAddress destip = (playerOnlineTable->getHeadAndModify(cmsg->getSubSpaceToCreate().getId(), (IPAddress) "127.0.0.1", -110)).getIp();
					if( destip != (IPAddress) "127.0.0.1" ) {
						sendMsg(*w, destip);
						w = notReadyMessages.erase(w);
					}
					else {
						break;
					}
				}
				else { ++w; }
			} else {
				++w;
			}
		}
		int xpos = intuniform(0,(subSpaceSize*gameSpaceSize)-1);
		int ypos = intuniform(0,(subSpaceSize*gameSpaceSize)-1);
		int subSpaceId =  ((ypos / subSpaceSize) * gameSpaceSize ) + (xpos / subSpaceSize);
		// (Create&)Get ResponsibleNode and neighbors
		std::vector<SubSpace*> neighbors = subSpaceTable->getAllNeighbors(subSpaceId);
		loginMsg->setResponsibleNodesArraySize( neighbors.size() );
		loginMsg->setServiceType(RESPONSE);
		loginMsg->setId( id );
		loginMsg->setXpos(xpos);
		loginMsg->setYpos(ypos);
		loginMsg->setSubSpaceSize( subSpaceSize );
		loginMsg->setGameSpaceSize( gameSpaceSize );
		bool notComplete = false;
		for(unsigned int i=0; i < neighbors.size(); i++) {
			loginMsg->setResponsibleNodes(i, *neighbors.at(i) );
			if( neighbors.at(i)->getRespIp() == (IPAddress) "127.0.0.1" ) {
				createResponsibleNode( neighbors.at(i)->getId() );
				notComplete = true;
			}
		}
		if(notComplete ) {
			notReadyMessages.push_back( loginMsg );
		} else {
			sendMsg(loginMsg, loginMsg->getSrcip() );
		}
	}
	else {
		//Confirmation
		// Delete Login from openLogins
		deleteFromDupedMessages( loginMsg);
		delete loginMsg;
		completedLogins++;
	}
}

void LobbyServer::processCreate(CreateNodeMessage* createMsg) {
	if( createMsg->getServiceType() == RESPONSE) {
		int subSpaceId = createMsg->getSubSpaceToCreate().getId();
		if( createMsg->getJobType() == CREATENODE) {
			switch ( createMsg->getNodeType() ) {
			case RESPONSIBLENODE: {
				if ( createMsg->getCreationSuccessfull() ) {
					// Set IP in the SubSpaceTable
					subSpaceTable->setResponsibleNode( subSpaceId,createMsg->getSrcip());
					// Inform Neighbor SubSpaces from the new IP
					informNeighbors( subSpaceTable->getSubSpace( subSpaceId ) );
					// Probably there are waiting some logins for the create
					addSubSpaceToOpenLogins( subSpaceId );
					//Perhaps there are waiting some request Ip's to
					sendWaitingRequests( subSpaceId, RESPONSIBLENODE );

					deleteFromDupedMessages(createMsg);

					//Create Backup Node for created Responsible Node
					CreateNodeMessage* backupCreate = new CreateNodeMessage("backupCreateFinishedResponsibleNodeCreate");
					backupCreate->setCommand(CREATE);
					backupCreate->setServiceType(REQUEST);
					backupCreate->setNodeType(BACKUPNODE);
					backupCreate->setJobType(CREATENODE);
					backupCreate->setSubSpaceToCreate( *(subSpaceTable->getSubSpace( subSpaceId )) );
					IPAddress destip = (playerOnlineTable->getHeadAndModify(subSpaceId, createMsg->getSrcip(), -110 )).getIp();
					if( destip != (IPAddress) "127.0.0.1" ){
						sendMsg(backupCreate, destip);
					}
					else {
						backupCreate->setSrcip(createMsg->getSrcip());
						thereIsANotReadyCreate = true;
						notReadyMessages.push_back(backupCreate);
					}
					delete createMsg;
				}
				else {
					// we have to create a new one
					delete createMsg;
				}
				break;
			}
			case BACKUPNODE: {
				subSpaceTable->getSubSpace(subSpaceId)->setBackupIp(createMsg->getSrcip());
				deleteFromDupedMessages(createMsg);
				delete createMsg;
				break;
			}
			case INTERMEDIATENODE: {
				deleteFromDupedMessages(createMsg);
				delete createMsg;
				break;
			}
			}
		}
		else if( createMsg->getJobType() == DELETENODE ) {
			playerOnlineTable->modify(subSpaceId, createMsg->getSrcip(), createMsg->getNodeType() );
			deleteFromDupedMessages(createMsg);
			delete createMsg;
		}
	}
	else {
		delete createMsg;
	}
}
		void processPingTimer();
void LobbyServer::processLogout(LogoutMessage* ymmog) {
	if( ymmog->getServiceType() == REQUEST ) {
		LogoutMessage* logoutMsg = (LogoutMessage*) (ymmog);
		logoutMsg->setServiceType(RESPONSE); //response
		logoutMsg->setSuccessfull( logout(logoutMsg->getId()) );
		sendMsg(logoutMsg, logoutMsg->getSrcip() );
	}
	else { // Confirmation
		delete ymmog;
	}
}


void LobbyServer::processRequestIp(RequestIpMessage* requestMsg) {

	if (requestMsg->getServiceType() == REQUEST ) {
		switch ( requestMsg->getNodeType() ) {
			case RESPONSIBLENODE: {
				if( subSpaceTable->getSubSpace( requestMsg->getSubSpaceId()) != NULL) {
					IPAddress requestedIp = subSpaceTable->getSubSpace( requestMsg->getSubSpaceId() )->getRespIp();
					requestMsg->setServiceType(RESPONSE);
					if ( requestedIp == (IPAddress) "127.0.0.1" ) {
						createResponsibleNode( requestMsg->getSubSpaceId() );
 						notReadyMessages.push_back(requestMsg);
					}
					else {
						requestMsg->setRequestedIp( requestedIp );
						sendMsg( requestMsg , requestMsg->getSrcip() );
					}
				}
				break;
			}
			default: {
				delete requestMsg;
			}
		}
	}
	else { //confirmation
		deleteFromDupedMessages( requestMsg );
		delete requestMsg;
	}
}


void LobbyServer::processUpdateRespIp(UpdateRespIpMessage* updateMsg) {
	if( updateMsg->getServiceType() == CONFIRMATION ) {
		deleteFromDupedMessages( updateMsg);
		delete updateMsg;
	}
}

void LobbyServer::processRequestNode(RequestNodeMessage*  requestMsg) {
	if( requestMsg->getServiceType() == REQUEST) {
		if ( !findOpenCreate(requestMsg->getSubSpaceId(), requestMsg->getNodeType() ) ) {
			//CreateNode
			CreateNodeMessage* createMsg = new CreateNodeMessage("CreateNodeCauseofRequest");
			createMsg->setCommand(CREATE);
			createMsg->setServiceType(REQUEST);
			createMsg->setNodeType( requestMsg->getNodeType() );
			createMsg->setJobType( CREATENODE );
			createMsg->setSubSpaceToCreate( *(subSpaceTable->getSubSpace(requestMsg->getSubSpaceId())) );

			if ( requestMsg->getNodeType() == BACKUPNODE ) {
				// Send the old one a delete...
				CreateNodeMessage* backupCreate = new CreateNodeMessage("BackupDeleteMsg");
				backupCreate->setCommand(CREATE);
				backupCreate->setServiceType(REQUEST);
				backupCreate->setNodeType(BACKUPNODE);
				backupCreate->setSubSpaceToCreate( *(subSpaceTable->getSubSpace(requestMsg->getSubSpaceId())) );
				backupCreate->setJobType(DELETENODE);
				sendMsg(backupCreate, subSpaceTable->getSubSpace(requestMsg->getSubSpaceId() )->getBackupIp() );
			}
			sendMsg( createMsg, (playerOnlineTable->getHeadAndModify(requestMsg->getSubSpaceId(), requestMsg->getSrcip(), -110 )).getIp() );
		}
		requestMsg->setServiceType(CONFIRMATION);
		sendMsg( requestMsg, requestMsg->getSrcip() );
	}
	else {
		delete requestMsg;
	}
}


void LobbyServer::processReplacement(ReplacementMessage* replaceMsg) {
	//send old responsible Node a delete msg
	CreateNodeMessage* deleteCreate = new CreateNodeMessage("deleteOldResponsibleNode");
	deleteCreate->setCommand(CREATE);
	deleteCreate->setServiceType(REQUEST);
	deleteCreate->setNodeType(RESPONSIBLENODE);
	deleteCreate->setJobType(DELETENODE);
	deleteCreate->setSubSpaceToCreate(*(subSpaceTable->getSubSpace(replaceMsg->getSubSpaceId())) );
	if( replaceMsg->getSrcip() != subSpaceTable->getSubSpace(replaceMsg->getSubSpaceId())->getRespIp() ) {
		sendMsg( deleteCreate, subSpaceTable->getSubSpace(replaceMsg->getSubSpaceId())->getRespIp() );
	}
	else { delete deleteCreate; }
	//missing
	subSpaceTable->setResponsibleNode(replaceMsg->getSubSpaceId() , replaceMsg->getSrcip() );
	informNeighbors( subSpaceTable->getSubSpace(replaceMsg->getSubSpaceId()) );
	// Get a new Backup Node
	CreateNodeMessage* backupCreate = new CreateNodeMessage( "backupCreateCauseOfReplacement" );
	backupCreate->setCommand(CREATE);
	backupCreate->setServiceType(REQUEST);
	backupCreate->setNodeType(BACKUPNODE);
	backupCreate->setJobType(CREATENODE);
	backupCreate->setSubSpaceToCreate(*(subSpaceTable->getSubSpace(replaceMsg->getSubSpaceId())) );
	sendMsg( backupCreate,(playerOnlineTable->getHeadAndModify(replaceMsg->getSubSpaceId(), replaceMsg->getSrcip(), -110 )).getIp() );
	//Confirm
	replaceMsg->setServiceType(CONFIRMATION);
	sendMsg(replaceMsg, replaceMsg->getSrcip() );
}

void LobbyServer::processStabilize(StabilizeMessage* stabilizeMsg) {
	SubSpace* subSpace = subSpaceTable->getSubSpace( stabilizeMsg->getSubSpaceId() );
	if( stabilizeMsg->getNodeType() == RESPONSIBLENODE) {
		if( subSpace->getRespIp() == stabilizeMsg->getSrcip() ) {
			stabilizeMsg->setCorrect(true);
		}
	}
	else if ( stabilizeMsg->getNodeType() == BACKUPNODE ) {
		if( subSpace->getBackupIp() == stabilizeMsg->getSrcip() ) {
			stabilizeMsg->setCorrect(true);
		}
	}
	stabilizeMsg->setSubSpace( *subSpace);
	stabilizeMsg->setServiceType(RESPONSE);
	sendMsg( stabilizeMsg, stabilizeMsg->getSrcip() );
}

void LobbyServer::processStabilizeTimer() {
	cancelEvent(stabilizeTimer);
	int i=0;
	for(i=0; i < gameSpaceSize*gameSpaceSize; i++) {
		StabilizeMessage* stabilizeMsg = new StabilizeMessage("Stabilze");
		stabilizeMsg->setCommand(STABILIZE);
		stabilizeMsg->setServiceType(REQUEST);
		stabilizeMsg->setSubSpaceId(i);
		stabilizeMsg->setNodeType(RESPONSIBLENODE);
		stabilizeMsg->setCorrect(true);
		stabilizeMsg->setSubSpace( *(subSpaceTable->getSubSpace( i )) );
		if ( subSpaceTable->getSubSpace(i)->getRespIp() != (IPAddress) "127.0.0.1" ) {
			sendMsg( stabilizeMsg, subSpaceTable->getSubSpace(i)->getRespIp() );
		} else { delete stabilizeMsg; }
	}
	scheduleAt(simTime() + stabilizeDelay, stabilizeTimer);
}


