//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//


#include <IPvXAddress.h>
#include "PlayerTable.h"
#include "YMMOGMessage_m.h"


Define_Module(PlayerTable);
void PlayerTable::initialize(int stage)
{
	// Wait for IPAdressResolver
	if ( stage != 5 ) return;
	// 
	subSpaceTable = check_and_cast<SubSpaceTable*>(simulation.moduleByPath("globalObserver.globalFunctions[0]"));

	player_autoincrement_id = 0;
	WATCH_LIST(playerTable);
	//WATCH_PTRLIST(nodeQueue);
}
PlayerTable::~PlayerTable() {
	this->finish();
}

void PlayerTable::handleMessage(cMessage* msg)
{
	error("this module doesn't handle messages");
}

void PlayerTable::finish()
{
	//no statistics yet
	//cleanup
	nodeQueue.clear();
	playerTable.clear();
}

int PlayerTable::add(int id, IPAddress ip, int Bandwith, int Cpu) {
	if( id == 0 ) {
		id = ++player_autoincrement_id;
	} 
	playerTable.push_back(PlayerNode(id, ip, Bandwith, Cpu));
	if( Bandwith >= 100 && nodeQueue.size() < (20 + playerTable.size() / 5)) {
		nodeQueue.push_front( &playerTable.back() );
	}
	return id;
}

bool PlayerTable::remove(int id) {
	// Delete from PlayerList
	for(n=nodeQueue.begin(); n != nodeQueue.end(); ++n) {	
		if ( (*n)->getId() == id ) {
			nodeQueue.erase(n);
		}
	}
	for(b=playerTable.begin(); b != playerTable.end(); ++b) {	
		if ( b->getId() == id ) {
			playerTable.erase(b);
			return true;
		}
	}
	return false;
}
void PlayerTable::remove(IPAddress ip) {
	// Delete from PlayerList
	for(n=nodeQueue.begin(); n != nodeQueue.end(); ++n) {	
		if ( (*n)->getIp() == ip ) {
			nodeQueue.erase(n);
			break;
		}
	}
	for(b=playerTable.begin(); b != playerTable.end(); ++b) {	
		if ( b->getIp() == ip ) {
			playerTable.erase(b);
			return;
		}
	}
	return;
}
int PlayerTable::getSize() {
	return playerTable.size();
}
PlayerNode PlayerTable::getHeadAndModify(int subSpaceId, IPAddress ip, int diff) {
	PlayerNode tmp = PlayerNode((IPAddress) "127.0.0.1");
	for(n=nodeQueue.begin(); n != nodeQueue.end(); ++n) {
		if( ( ! subSpaceTable->isInNodeList(subSpaceId, (*n)->getIp() ) ) && ip != (*n)->getIp() ) {
				subSpaceTable->addNode(subSpaceId, (*n)->getIp() );
				PlayerNode* tmp = *n;
				tmp->modifyBandwith(diff);
				nodeQueue.erase(n);
				if(tmp->getBandwith() > 100 && nodeQueue.size() < (10 + ( playerTable.size() / 5 )) ) {
					nodeQueue.push_back(tmp);
				}
				return *tmp;
		}
	}
	//If all choosen Nodes in BackupNodeQueue already are nodes of this subSpace, choose one out of all players.
	playerTable.sort();
	for(b=playerTable.begin(); b != playerTable.end(); ++b) {
		if( b->getIp() != ip && !subSpaceTable->isInNodeList(subSpaceId, (*n)->getIp() ) ) {
			if( b->getBandwith() < abs(diff) ) {
				return tmp;
			}
			(*b).modifyBandwith(diff);
			return *b;
		}
	}
	return tmp;
}

void PlayerTable::modify( int subSpaceId, IPAddress playerIp, int nodeType) {
	if( nodeType == INTERMEDIATENODE) {
		subSpaceTable->removeNode(subSpaceId, playerIp);
		PlayerNode* player = findPlayerNodeByIp(playerIp);
		if( player != NULL ) {
			player->modifyBandwith(+100);
		}
	}
}

PlayerNode* PlayerTable::findPlayerNodeByIp(IPAddress playerIp) {
	for( b = playerTable.begin() ; b != playerTable.end() ; ++b) {
		if( b->getIp() == playerIp) {
			return &*b;
		}
	}
	return NULL;
}
