//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
#include "ResponsibleNodeData.h"


ResponsibleNodeData::~ResponsibleNodeData() {
	// Free Playerlist
	//delete tree;
	// Free eventlist
	eventlist.clear();
}

ResponsibleNodeData::ResponsibleNodeData(SubSpace subSpace , int maxPlayerLeafs , int treeWidth, double time) {
	this->subSpace = subSpace;
	this->maxPlayerLeafs = maxPlayerLeafs;
	this->treeWidth = treeWidth;
	this->last_timeslot = time;
	this->tree = new LoadBalancingTree( subSpace.getId(), maxPlayerLeafs, treeWidth );
	gamestate = GameState();
	dropped_events=0;
	eventCount = 0;
}

ResponsibleNodeData::ResponsibleNodeData(SubSpace subSpace , int maxPlayerLeafs , int treeWidth, double time, std::list<TreeNode> tree) {
	this->subSpace = subSpace;
	this->maxPlayerLeafs = maxPlayerLeafs;
	this->treeWidth = treeWidth;
	this->last_timeslot = time;
	gamestate = GameState();
	dropped_events=0;
	eventCount = 0;
	this->tree = new LoadBalancingTree( subSpace.getId(), maxPlayerLeafs, treeWidth );
	this->tree->rebuildTree(tree);
};

std::list<TreeMessage> ResponsibleNodeData::subscribe(PlayerNode player) {
	/*std::list<TreeMessage> tmp;
	for(tmpListIt=tmpList.begin(); tmpListIt != tmpList.end(); ++tmpListIt) {
		if(tmpListIt->getId() == player.getId() ) {
			return tmp;
		}
	}
	tmpList.push_back(player);
	return tmp;*/
	return tree->addPlayer(player.getId() , player.getIp() );
}

std::list<TreeMessage> ResponsibleNodeData::unsubscribe(int playerId) {
	/*
	std::list<TreeMessage> tmp;
	for(tmpListIt=tmpList.begin(); tmpListIt != tmpList.end(); ++tmpListIt) {
			if(tmpListIt->getId() == playerId) {
				tmpList.erase(tmpListIt);
				return tmp;
		}
	}
	return tmp;
	*/
	std::list<TreeMessage> msgList = tree->removePlayer(playerId);
	return msgList;
}

std::list<TreeMessage> ResponsibleNodeData::unsubscribe(IPAddress playerIp) {
/*	std::list<TreeMessage> tmp;
	for(tmpListIt=tmpList.begin(); tmpListIt != tmpList.end(); ++tmpListIt) {
			if(tmpListIt->getIp() == playerIp) {
				tmpList.erase(tmpListIt);
				return tmp;
		}
	}
	return tmp;
*/
	std::list<TreeMessage> msgList = tree->removePlayer(playerIp);
	return msgList;
}

std::list<TreeNode> ResponsibleNodeData::getPlayerList() {
	return tree->serialize() ;
}
std::list<IPAddress> ResponsibleNodeData::getNodeList() {
	return tree->getAllNodes();
}
std::list<IPAddress> ResponsibleNodeData::getAllIntNodes() {
	return tree->getAllIntNodes();
}
void ResponsibleNodeData::setSubSpace( SubSpace subSpace) {
	if( this->subSpace.getId() == subSpace.getId() ) {
		this->subSpace = subSpace;
	}
}
void ResponsibleNodeData::addEvent(Event event) {
	eventCount++;
	if( event.getTimeStamp() > last_timeslot ) {
		eventlist.push_back(event);
	}
	else { dropped_events++; }
}

int ResponsibleNodeData::getEventListSize() {
	return eventlist.size();
}

std::vector<Event> ResponsibleNodeData::endTimeSlot(double now) {
	std::vector<Event> ordered_eventlist;
	eventlist.sort();
	while( eventlist.size() > 0 ) {
		ordered_eventlist.push_back( eventlist.front() );
		eventlist.pop_front();
	}
	ordered_eventlist.push_back(Event(now, -1 , NPC, 0 , 0 ) );
	subSpace.setTimeSlot( subSpace.getTimeSlot() + 1 );
	last_timeslot = now;
	return ordered_eventlist;
}


SubSpace ResponsibleNodeData::getSubSpace() {
	return subSpace;
}

int ResponsibleNodeData::getPlayerListSize() {
	return tree->size;
}
 
std::list<IPAddress> ResponsibleNodeData::getList() {
	return tree->getList();
}

void ResponsibleNodeData::setNeighbor(int direction, IPAddress resp_ip) {
	this->subSpace.setNeighbor(direction, resp_ip);
}


int ResponsibleNodeData::getId() {
	return this->subSpace.getId();
}

void ResponsibleNodeData::setBackupIp(IPAddress backupIp) {
	this->subSpace.setBackupIp(backupIp);
}
IPAddress ResponsibleNodeData::getBackupIp() {
	return this->subSpace.getBackupIp();
}
	
std::list<TreeMessage> ResponsibleNodeData::addIntermediateNode(IPAddress intermediateIp ) {
	return tree->addIntermediateNode(intermediateIp);	
}

std::list<IPAddress> ResponsibleNodeData::replaceIntermediateNode(IPAddress oldip, IPAddress newip) {
	return tree->exchangeNode(oldip, newip);
}
int ResponsibleNodeData::getReplacedIntermediateNodeCount() {
	return tree->replacedIntermediateNodes;
}
double ResponsibleNodeData::getDroppedEvents() {
	double ret = -1;
	if (eventCount > 0) {
		ret = (double) dropped_events / eventCount;
	}
	eventCount = 0;
	dropped_events = 0;
	return ret;
}

std::ostream& operator<<(std::ostream& Stream, ResponsibleNodeData r)
{
    return Stream << "ID:" << r.getId() << " Players: " << r.tree->size << " Events: " << r.getEventListSize() << " Dropped Events " << r.getDroppedEvents() << " MaxPlayerLeafs " << r.maxPlayerLeafs << " Real Intermediate Size: " <<  "\nNeighbors: " << r.getSubSpace() << r.tree ;
}
