//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.#endif
//


#include <IPvXAddress.h>
#include "SubSpaceTable.h"
#include <iostream>


Define_Module(SubSpaceTable);
void SubSpaceTable::initialize(int stage)
{
	// Wait for IPAdressResolver
	if ( stage != 5 ) return;
	// 
	WATCH_PTRVECTOR(subSpaceTable);
	//WATCH_VECTOR(nodeList);
	
	collectStatsTimer = new cMessage("CollectStats");
	collectStatsDelay = 1;
	scheduleAt(simTime() + collectStatsDelay, collectStatsTimer);
	
	sessionTime.setName("Session Stats");
	//initialize eventlog
	int i =0;
	eventCount = 0;
	while ( i < 1000 ) {
		eventLog[i] = 0;
		i++;
	}
}

SubSpaceTable::~SubSpaceTable() {
	this->finish();
}

void SubSpaceTable::handleMessage(cMessage* msg)
{
	if( msg->isName("CollectStats") ) {
		processCollectStats();
	} else {
		error("this module doesn't handle messages");
	}
}

void SubSpaceTable::finish()
{
	//cleanup
	for(uint i=0; i < subSpaceTable.size(); ++i) {
		delete subSpaceTable.at(i);
		/*delete maxEventDelayLog.at(i);
		delete avgEventDelayLog.at(i);
		delete minEventDelayLog.at(i);
		*/
		delete playerOnlineLog.at(i);
		delete intNodesCountLog.at(i);
		/*
		delete replacedNodesLog.at(i);
		delete maxCommunicationLog.at(i);
		delete avgCommunicationLog.at(i);
		delete minCommunicationLog.at(i);
		*/
	}
	if (backupLogUp != NULL) {
		delete backupLogUp;
		backupLogUp = NULL;
		delete backupLogDown;
		delete respLogUp;
		delete respLogDown;
		delete intLogUp;
		delete intLogDown;
		delete treeLog;
		subSpaceTable.clear();
		int i=0;
		unsigned long count=0;
		while(i < 1000) {
			count += eventLog[i];
			eventOut->record(i, (double) count / eventCount );
			i++;
		}
		delete eventOut;
	}
	
	//communicationStats.clear();
	// no statistics yet
}

void SubSpaceTable::createTable(int size) {
	this->size = size;
	for(int id=0; id < size*size; id++ ) {
		subSpaceTable.push_back( new SubSpace(id, (IPAddress) "127.0.0.1" , (IPAddress) "127.0.0.1" ) );
		replacedNodesLog.push_back(0);
		std::list<IPAddress> newList;
		nodeList.push_back( newList );
		
		char buffer[255];
		/*
		// MaxEventDelay
		cOutVector* log = new cOutVector;
		maxEventDelayLog.push_back( log  );
		sprintf(buffer, "MaxEventDelayLog for SubSpace ID: %i", id);
		maxEventDelayLog.back()->setName(buffer);
		// AvgEventDelay
		log = new cOutVector;
		avgEventDelayLog.push_back( log  );
		sprintf(buffer, "AvgEventDelayLog for SubSpace ID: %i", id);
		avgEventDelayLog.back()->setName(buffer);
		// MinEventDelay
		log = new cOutVector;
		minEventDelayLog.push_back( log  );
		sprintf(buffer, "MinEventDelayLog for SubSpace ID: %i", id);
		minEventDelayLog.back()->setName(buffer);		
		*/
		// PlayerOnline
		cOutVector* log = new cOutVector;
		playerOnlineLog.push_back( log  );
		sprintf(buffer, "PlayerOnline for SubSpace ID: %i", id);
		playerOnlineLog.back()->setName(buffer);
		/*// Dropped Events
		log = new cOutVector;
		droppedEventsLog.push_back( log  );
		sprintf(buffer, "Dropped Events for SubSpace ID: %i", id);
		droppedEventsLog.back()->setName(buffer);
		
		// Replaced IntermediateNodes
		log = new cOutVector;
		replacedNodesLog.push_back( log  );
		sprintf(buffer, "Replacements of IntermediateNodes for SubSpace ID: %i", id);
		replacedNodesLog.back()->setName(buffer);
		*/
		// # of IntermediateNodes 
		log = new cOutVector;
		intNodesCountLog.push_back( log  );
		sprintf(buffer, "Number of actual IntermediateNodes for SubSpace ID: %i", id);
		intNodesCountLog.back()->setName(buffer);
		/*// Max Communication Delay
		sprintf(buffer, "Max Communication Delay for SubSpace ID: %i", id);
		log = new cOutVector(buffer,2);
		maxCommunicationLog.push_back( log  );
		// Avg Communication Delay
		sprintf(buffer, "Avg Communication Delay for SubSpace ID: %i", id);
		log = new cOutVector(buffer,2);
		avgCommunicationLog.push_back( log  );
		// Min Communication Delay
		sprintf(buffer, "Min Communication Delay for SubSpace ID: %i", id);		
		log = new cOutVector(buffer, 2);
		minCommunicationLog.push_back( log  );
		cStdDev stat;
		communicationStats.push_back(stat);*/
	}
	//maxEventDelay.setName("Max EventDelay for all Subspace/Players:");
	//avgEventDelay.setName("Avg EventDelay for all Subspace/Players:");
	//minEventDelay.setName("Min EventDelay for all SubSpace/Players:");
	
	//maxTreeDelay.setName("Max TreeDelay for all Subspace/Players:");
	//avgTreeDelay.setName("Avg TreeDelay for all Subspace/Players:");
	//minTreeDelay.setName("Min TreeDelay for all SubSpace/Players:");
	droppedEventsLog.setName("Dropped Events Logs for all Subspaces");
	treeLog = new cOutVector("TreeDelay for all Subspace/Players", 2);
	respLogUp = new cOutVector("ResponsibleNode Upstream Bandwith Log", 2);
	respLogDown = new cOutVector("ResponsibleNode Downstream Bandwith Log", 2);
	backupLogUp = new cOutVector("BackupNode Upstream Bandwith Log", 2);
	backupLogDown = new cOutVector("BackupNode Downstream Bandwith Log", 2);
	intLogUp = new cOutVector("IntermediateNode Upstream Bandwith Log", 2);
	intLogDown = new cOutVector("IntermediateNode Downstream Bandwith Log", 2);
	eventOut = new cOutVector("EventDelay Log for all Subspace/Players", 2);
	//eventLog.setName("EventDelay Log for all Subspace/Players");
	
}
void SubSpaceTable::record(int subSpaceId, int logType, double value) {
	/*if( logType == 0 ) {
		maxEventDelayLog.at(subSpaceId)->record(value);
	}
	else if( logType == 1) {
		avgEventDelayLog.at(subSpaceId)->record(value);
	}
	else if ( logType == 2) {
		minEventDelayLog.at(subSpaceId)->record(value);
	}
	*/
	if ( logType == 5) {
		replacedNodesLog[(int)subSpaceId] = value;
/*		maxCommunicationLog.at(subSpaceId)->record(communicationStats.at(subSpaceId).max(), value);
		avgCommunicationLog.at(subSpaceId)->record(communicationStats.at(subSpaceId).mean(), value);
		minCommunicationLog.at(subSpaceId)->record(communicationStats.at(subSpaceId).min(), value);
		communicationStats.at(subSpaceId).clearResult();
		*/
	}
	else if ( logType == 3) {
		playerOnlineLog.at(subSpaceId)->record(value);
	}
	else if ( logType == 4) {
		if( value >= 0) droppedEventsLog.record((double) value);
	}
	else if ( logType == 6) {
		intNodesCountLog.at(subSpaceId)->record(value);
	}
	/*else if( logType == 7 ) {
		communicationStats.at(subSpaceId).collect(value);
	}*/
	else if( logType == 8 ) {
		sessionTime.record(value);
	}
	else if( logType == 9 ) {
		//eventDelay.collect(value);
		int tmp=0;
		tmp = value*1000;
		if( tmp > 999 ) tmp = 999;
		eventLog[tmp]++;
		eventCount++;
	}
	else if( logType == 10 ) {
		//treeDelay.collect(value);
		treeLog->record(value,replacedNodesLog.at(subSpaceId));
	}
	else if( logType == 11 ) {
		//subSpaceId = |player(r)| 
		respLogUp->record(value, subSpaceId);
	}
	else if( logType == 12 ) {
		//subSpaceId = |player(r)| 
		respLogDown->record(value, subSpaceId);
	}
	else if( logType == 13 ) {
		//subSpaceId = |player(r)| 
		backupLogUp->record(value, subSpaceId);
	}
	else if( logType == 14 ) {
		//subSpaceId = |player(r)| 
		backupLogDown->record(value, subSpaceId);
	}
	else if( logType == 15 ) {
		//subSpaceId = |player(r)| 
		intLogUp->record(value, subSpaceId);
	}
	else if( logType == 16 ) {
		//subSpaceId = |player(r)| 
		intLogDown->record(value, subSpaceId);
	}		/*else if ( logType == 7) {
		maxCommunicationLog.at(subSpaceId)->record(value);
	}
	else if ( logType == 8) {
		avgCommunicationLog.at(subSpaceId)->record(value);
	}
	else if ( logType == 9) {
		minCommunicationLog.at(subSpaceId)->record(value);
	}
	*/

}
int SubSpaceTable::getSize() {
	return subSpaceTable.size();
}

void SubSpaceTable::toString() {
	for(unsigned int i=0; i < subSpaceTable.size(); i++) {
		if( subSpaceTable.at(i)->getRespIp() != "127.0.0.1" ) {
			if( (i % (int) sqrt(subSpaceTable.size()) ) == 0 ) {
				ev << "\n";
			}
			ev << "x";
		}
		else { 
		if( (i % (int) sqrt(subSpaceTable.size()) ) == 0 ) {
				ev << "\n";
			}
		ev << "-";
		}
	}
}

SubSpace* SubSpaceTable::getSubSpace(unsigned int id) {
	if( id < subSpaceTable.size() ) {
		return subSpaceTable.at(id);
	}
	return NULL;
}

void SubSpaceTable::setResponsibleNode(int id, IPAddress resp_ip) {
	subSpaceTable.at(id)->setRespIp(resp_ip);
	
	if ( id == 0 ) { // left upper corner
		subSpaceTable.at(id + 1)->setNeighbor(W, resp_ip);
		subSpaceTable.at(id + size)->setNeighbor(N, resp_ip);
		subSpaceTable.at(id + size + 1)->setNeighbor(NW, resp_ip);
		return;
	}
	if ( id == (size-1) ) { // right upper corner
		subSpaceTable.at(id -1 )->setNeighbor(E, resp_ip);
		subSpaceTable.at(id + size -1 )->setNeighbor(NE, resp_ip);
		subSpaceTable.at(id + size)->setNeighbor(N, resp_ip);
		return;
	}
	if ( id == (size*size) - size ) { // left buttom corner
		subSpaceTable.at(id - size)->setNeighbor(S, resp_ip);
		subSpaceTable.at(id - size +1)->setNeighbor(SW, resp_ip);
		subSpaceTable.at(id + 1)->setNeighbor(W, resp_ip);
		return;
	}
	if ( id == (size*size) - 1) { // right buttom corner
		subSpaceTable.at(id - size - 1)->setNeighbor(SE, resp_ip);
		subSpaceTable.at(id - size)->setNeighbor(S, resp_ip);
		subSpaceTable.at(id - 1)->setNeighbor(E, resp_ip);
		return;
	}
	if ( id < size ) { // First Row, we got no top neighbors
		subSpaceTable.at(id -1 )->setNeighbor(E, resp_ip);
		subSpaceTable.at(id + 1)->setNeighbor(W, resp_ip);
		subSpaceTable.at(id + size -1 )->setNeighbor(NE, resp_ip);
		subSpaceTable.at(id + size)->setNeighbor(N, resp_ip);
		subSpaceTable.at(id + size + 1)->setNeighbor(NW, resp_ip);
		return;
	}
	if ( (id % size) == 0 ) { // Left Row -> we got no left neighbors
		subSpaceTable.at(id - size)->setNeighbor(S, resp_ip);
		subSpaceTable.at(id - size +1)->setNeighbor(SW, resp_ip);
		subSpaceTable.at(id + 1)->setNeighbor(W, resp_ip);
		subSpaceTable.at(id + size)->setNeighbor(N, resp_ip);
		subSpaceTable.at(id + size + 1)->setNeighbor(NW, resp_ip);
		return;
	}
	if ( (id % size) == (size - 1) ) { // Right Row -> we got no right neighbors
		subSpaceTable.at(id - size - 1)->setNeighbor(SE, resp_ip);
		subSpaceTable.at(id - size)->setNeighbor(S, resp_ip);
		subSpaceTable.at(id -1 )->setNeighbor(E, resp_ip);
		subSpaceTable.at(id + size -1 )->setNeighbor(NE, resp_ip);
		subSpaceTable.at(id + size)->setNeighbor(N, resp_ip);
		return;
	}
	if ( id >= (size*size) - size ) { // Buttom Row -> we got no buttom neighbors
		subSpaceTable.at(id - size - 1)->setNeighbor(SE, resp_ip);
		subSpaceTable.at(id - size)->setNeighbor(S, resp_ip);
		subSpaceTable.at(id - size +1)->setNeighbor(SW, resp_ip);
		subSpaceTable.at(id -1 )->setNeighbor(E, resp_ip);
		subSpaceTable.at(id + 1)->setNeighbor(W, resp_ip);
		return;
	}
	// We have really all 8 neighbors:
	subSpaceTable.at(id - size - 1)->setNeighbor(SE, resp_ip);
	subSpaceTable.at(id - size)->setNeighbor(S, resp_ip);
	subSpaceTable.at(id - size +1)->setNeighbor(SW, resp_ip);
	subSpaceTable.at(id -1 )->setNeighbor(E, resp_ip);
	subSpaceTable.at(id + 1)->setNeighbor(W, resp_ip);
	subSpaceTable.at(id + size -1 )->setNeighbor(NE, resp_ip);
	subSpaceTable.at(id + size)->setNeighbor(N, resp_ip);
	subSpaceTable.at(id + size + 1)->setNeighbor(NW, resp_ip);
}

void SubSpaceTable::setBackupNode(int id, IPAddress backup_ip) {
	subSpaceTable.at(id)->setBackupIp(backup_ip);
}

std::vector<SubSpace*> SubSpaceTable::getAllNeighbors(int id) {
	// returns all 8+1 Neighbors of a Subspace.push_back(
	std::vector<SubSpace*> neighbors;
	neighbors.push_back(getSubSpace(id)); // Not a true neighbor but we want to give it back too
	// Some special Cases we got no 8 neighbors
	if ( id == 0 ) { // left upper corner
		neighbors.push_back(getSubSpace( id + 1));
		neighbors.push_back(getSubSpace( id + size));
		neighbors.push_back(getSubSpace( id + size + 1));
		return neighbors;
	}
	else if ( id == (size-1) ) { // right upper corner
		neighbors.push_back(getSubSpace( id - 1));
		neighbors.push_back(getSubSpace( id + size - 1));
		neighbors.push_back(getSubSpace( id + size));
		return neighbors;
	}
	else if ( id == (size*size) - size ) { // left buttom corner
		neighbors.push_back(getSubSpace( id-size ));
		neighbors.push_back(getSubSpace( id - size + 1));
		neighbors.push_back(getSubSpace( id + 1));
		return neighbors;
	}
	else if ( id == (size*size) - 1) { // right buttom corner
		neighbors.push_back(getSubSpace( id - size- 1));
		neighbors.push_back(getSubSpace( id-size ));
		neighbors.push_back(getSubSpace( id - 1));
		return neighbors;
	}
	
	else if ( id < size ) { // First Row, we got no top neighbors
		neighbors.push_back(getSubSpace( id - 1));
		neighbors.push_back(getSubSpace( id + 1));
		neighbors.push_back(getSubSpace( id + size - 1));
		neighbors.push_back(getSubSpace( id + size));
		neighbors.push_back(getSubSpace( id + size + 1));
		return neighbors;
	}
	else if ( (id % size) == 0 ) { // Left Row -> we got no left neighbors
		neighbors.push_back(getSubSpace( id-size ));
		neighbors.push_back(getSubSpace( id - size + 1));
		neighbors.push_back(getSubSpace( id + 1));
		neighbors.push_back(getSubSpace( id + size));
		neighbors.push_back(getSubSpace( id + size + 1));
		return neighbors;
	}
	else if ( (id % size) == (size - 1) ) { // Right Row -> we got no right neighbors
		neighbors.push_back(getSubSpace( id - size- 1));
		neighbors.push_back(getSubSpace( id-size ));
		neighbors.push_back(getSubSpace( id - 1));
		neighbors.push_back(getSubSpace( id + size - 1));
		neighbors.push_back(getSubSpace( id + size));
		return neighbors;
	}
	else if ( id >= (size*size) - size ) { // Buttom Row -> we got no buttom neighbors
		neighbors.push_back(getSubSpace( id - size- 1));
		neighbors.push_back(getSubSpace( id-size ));
		neighbors.push_back(getSubSpace( id - size + 1));
		neighbors.push_back(getSubSpace( id - 1));
		neighbors.push_back(getSubSpace( id + 1));
		return neighbors;
	}
	// We really got 8 neighbors:
	neighbors.push_back(getSubSpace( id - size- 1));
	neighbors.push_back(getSubSpace( id-size ));
	neighbors.push_back(getSubSpace( id - size + 1));
	neighbors.push_back(getSubSpace( id - 1));
	neighbors.push_back(getSubSpace( id + 1));
	neighbors.push_back(getSubSpace( id + size - 1));
	neighbors.push_back(getSubSpace( id + size));
	neighbors.push_back(getSubSpace( id + size + 1));
	return neighbors;

}


void SubSpaceTable::addNode(int subSpaceId, IPAddress nodeIp) {
	nodeList.at(subSpaceId).push_back(nodeIp);
}
void SubSpaceTable::removeNode(int subSpaceId, IPAddress nodeIp) {
	std::list<IPAddress>::iterator i;
	for( i = nodeList.at(subSpaceId).begin(); i != nodeList.at(subSpaceId).end(); ++i) {
		if( *i == nodeIp) {
			nodeList.at(subSpaceId).erase(i);
		}
		return;
	}
}
bool SubSpaceTable::isInNodeList(int subSpaceId, IPAddress nodeIp) {
	std::list<IPAddress>::iterator i;
	for( i = nodeList.at(subSpaceId).begin(); i != nodeList.at(subSpaceId).end(); ++i) {
		if( *i == nodeIp) {
			return true;
		}
	}
	return false;
}
void SubSpaceTable::processCollectStats() {
	cancelEvent(collectStatsTimer);
	/*maxEventDelay.record(eventDelay.max());
	avgEventDelay.record(eventDelay.mean());
	minEventDelay.record(eventDelay.min());
	maxTreeDelay.record(treeDelay.max());
	avgTreeDelay.record(treeDelay.mean());
	minTreeDelay.record(treeDelay.min());
	treeDelay.clearResult();
	eventDelay.clearResult();
	scheduleAt(simTime()+ collectStatsDelay, collectStatsTimer);*/
}
