#include "YMMOGBase.h"

void YMMOGBase::initialize(int stage) {
	if ( stage != 5 )
		return;

	debug = par("debug");
	localPort = par("localPort");

	thisNode.ip = IPAddressResolver().addressOf(parentModule()->parentModule()).get4();
	thisNode.port = localPort;
	bootstrapOracle = BootstrapOracleAccess().get();
	globalStatistics = GlobalStatisticsAccess().get();
	// set up UDP
	cMessage* msg = new cMessage("UDP_C_BIND", UDP_C_BIND);
	UDPControlInfo* ctrl = new UDPControlInfo();
	ctrl->setSrcPort(localPort);
	ctrl->setSockId(UDPSocket::generateSocketId());
	msg->setControlInfo(ctrl);
	send(msg, "to_udp");

	resendTimer = new cMessage("resendTimer");
	stabilizeTimer = new cMessage("stabilizeTimer");
	bandwithTimer = new cMessage("bandwithTimer");
	stabilizeDelay = 30;
	resendDelay = 1.5;
	selfMessages = 0;
	bandwithDelay = 60;
	scheduleAt(simTime() + resendDelay, resendTimer);
	scheduleAt(simTime() + stabilizeDelay, stabilizeTimer);
	scheduleAt(simTime() + bandwithDelay, bandwithTimer);
	initializeStats();
	initialize();
}
void YMMOGBase::initialize() {
	ev << "You should add your own initialize()";
}

void YMMOGBase::initializeStats() {
	WATCH_PTRLIST( dupedMessages );
	WATCH_PTRLIST(notReadyMessages);
	WATCH(selfMessages);
	WATCH_PTRLIST(pings);

	droppedMessages=0;
	sentMessages=0;
	receivedMessages=0;
	WATCH(sentMessages);
	WATCH(receivedMessages);
	WATCH(droppedMessages);

	sentSum = 0;
	receivedSum = 0;
	WATCH(sentSum);
	WATCH(receivedSum);

	upstream = new cOutVector("Used Upstream");
	downstream = new cOutVector("Used Downstream");
	up=0;
	down=0;
	sessionBegin = simTime();


	messageBytes = 0;
	overheadBytes = 0;
	sentOverheadBytes = 0;
	receivedOverheadBytes = 0;
	sentBytes = 0;
	receivedBytes = 0;

	// Sent Variables:
	sentLogins = 0;
	sentLogouts = 0;
	sentSubscribes = 0;
	sentUnSubscribes = 0;
	sentEvents = 0;
	sentCreates = 0;
	sentRequestIps = 0;
	sentUpdateRespIps = 0;
	sentEventLists = 0;
	sentPings = 0;
	sentPlayerLists = 0;
	sentRequestNodes = 0;
	sentReplacements = 0;
	sentDelays = 0;
	sentReplaceInt = 0;
	sentUndefined = 0;

	WATCH( sentLogins );
	WATCH( sentLogouts );
	WATCH( sentSubscribes );
	WATCH( sentUnSubscribes );
	WATCH( sentEvents );
	WATCH( sentCreates );
	WATCH( sentRequestIps );
	WATCH( sentUpdateRespIps );
	WATCH( sentEventLists );
	WATCH( sentPings );
	WATCH( sentPlayerLists );
	WATCH( sentRequestNodes );
	WATCH( sentReplacements );
	WATCH( sentDelays );
	WATCH( sentReplaceInt );
	WATCH( sentUndefined );

	// Received Variables:
	receivedLogins = 0;
	receivedLogouts = 0;
	receivedSubscribes = 0;
	receivedUnSubscribes = 0;
	receivedEvents = 0;
	receivedCreates = 0;
	receivedRequestIps = 0;
	receivedUpdateRespIps = 0;
	receivedEventLists = 0;
	receivedPings = 0;
	receivedPlayerLists = 0;
	receivedRequestNodes = 0;
	receivedReplacements = 0;
	receivedDelays = 0;
	receivedReplaceInt = 0;

	WATCH( receivedLogins );
	WATCH( receivedLogouts );
	WATCH( receivedSubscribes );
	WATCH( receivedUnSubscribes );
	WATCH( receivedEvents );
	WATCH( receivedCreates );
	WATCH( receivedRequestIps );
	WATCH( receivedUpdateRespIps );
	WATCH( receivedEventLists );
	WATCH( receivedRequestNodes );
	WATCH( receivedReplacements );
	WATCH( receivedDelays );
	WATCH( receivedReplaceInt );
}
void YMMOGBase::finish() {
	finishStats();
	ev << "You should add your own finish()";
}
void YMMOGBase::finishStats() {

	globalStatistics->addStdDev("Total Overhead:", overheadBytes);
	globalStatistics->addStdDev("sentOverhead:", sentOverheadBytes);
	globalStatistics->addStdDev("receivedOverhead:", receivedOverheadBytes);
	globalStatistics->addStdDev("Total Bytes:", messageBytes);
	globalStatistics->addStdDev("Total sent Bytes:", sentBytes);
	globalStatistics->addStdDev("Total received Bytes:", receivedBytes);
	if( sentOverheadBytes > 0) globalStatistics->addStdDev("Sent Overhead in %", (double) sentOverheadBytes / sentBytes);
	if( receivedOverheadBytes > 0) globalStatistics->addStdDev("Received Overhead in %", (double) receivedOverheadBytes / receivedBytes);
	globalStatistics->addStdDev("Avg Bandwith Usage:", (double) messageBytes / (simTime() - sessionBegin) );
	globalStatistics->addStdDev("DroppedMessages:", droppedMessages);


	/** Sents **/

	globalStatistics->addStdDev("SentLogins: ", sentLogins );
	globalStatistics->addStdDev("SentLogouts: ", sentLogouts );
	globalStatistics->addStdDev("SentSubscribes: ", sentSubscribes );
	globalStatistics->addStdDev("SentUnSubscribes: ", sentUnSubscribes );
	globalStatistics->addStdDev("SentEvents: ", sentEvents );
	globalStatistics->addStdDev("SentCreates: ", sentCreates );
	globalStatistics->addStdDev("SentRequestIps: ", sentRequestIps );
	globalStatistics->addStdDev("SentUpdates: ", sentUpdateRespIps );
	globalStatistics->addStdDev("SentEventLists: ", sentEventLists );
	globalStatistics->addStdDev("SentPings: ", sentPings );
	globalStatistics->addStdDev("SentPlayerLists: ", sentPlayerLists );
	globalStatistics->addStdDev("SentRequestNodes: ", sentRequestNodes );
	globalStatistics->addStdDev("SentReplacements: ", sentReplacements );
	globalStatistics->addStdDev("SentDelays: ", sentDelays );
	globalStatistics->addStdDev("SentReplaceInt: ", sentReplaceInt );
	globalStatistics->addStdDev("SentUndefined: ", sentUndefined );

	/** Received **/

	globalStatistics->addStdDev("ReceivedLogins: ", receivedLogins );
	globalStatistics->addStdDev("ReceivedLogouts: ", receivedLogouts );
	globalStatistics->addStdDev("ReceivedSubScribes: ", receivedSubscribes );
	globalStatistics->addStdDev("ReceivedUnSubscribes: ", receivedUnSubscribes );
	globalStatistics->addStdDev("ReceivedEvents: ", receivedEvents );
	globalStatistics->addStdDev("ReceivedCreates: ", receivedCreates );
	globalStatistics->addStdDev("ReceivedRequestIps: ", receivedRequestIps );
	globalStatistics->addStdDev("ReceivedUpdateRespIps: ", receivedUpdateRespIps );
	globalStatistics->addStdDev("ReceivedEventLists: ", receivedEventLists );
	globalStatistics->addStdDev("ReceivedRequestNodes: ", receivedRequestNodes );
	globalStatistics->addStdDev("ReceivedReplacements: ", receivedReplacements );
	globalStatistics->addStdDev("ReceivedDelays: ", receivedDelays );
	globalStatistics->addStdDev("ReceivedReplaceInt: ", receivedReplaceInt );


	// cleanup
	while( notReadyMessages.size() > 0) {
		delete notReadyMessages.front();
		notReadyMessages.pop_front();
	}
	notReadyMessages.clear();
	while( dupedMessages.size() > 0 ) {
		delete dupedMessages.front();
		dupedMessages.pop_front();
	}
	dupedMessages.clear();
	while (pings.size() > 0 ) {
		delete pings.front();
		pings.pop_front();
	}

	if( pingTimer != NULL ) {
		cancelEvent(pingTimer);
		delete pingTimer;
		pingTimer = NULL;
		cancelEvent(resendTimer);
		delete resendTimer;
		cancelEvent(bandwithTimer);
		delete bandwithTimer;
		delete upstream;
		delete downstream;
	}
}

void YMMOGBase::handleMessage(cMessage* msg) {
	processSelfTimers(msg);
	if ( msg->arrivedOn("from_udp") )
	{
		receivedMessages++;
		receivedBytes += msg->byteLength();
		down += msg->byteLength();
		// remove udp interface control information
		UDPControlInfo* udpControlInfo = check_and_cast<UDPControlInfo*>(msg->removeControlInfo());
		delete udpControlInfo;
		YMMOGMessage* ymmog = dynamic_cast<YMMOGMessage*>(msg);
		if ( ymmog != NULL )
		{
			if( ymmog->getTimeStamp() < simTime() - 3 ) {
				delete msg;
				return;
			}
			// Here we will process the diffrent messages
			switch ( ymmog->getCommand() ) {
				case LOGIN: { // Login Response
					LoginMessage* loginMsg = dynamic_cast<LoginMessage*>(ymmog);
					if( loginMsg != NULL ) {
						processLogin(loginMsg);
					}
					break;
				}
				case LOGOUT: { // Logout Response
					LogoutMessage *logoutMsg = dynamic_cast<LogoutMessage*>(ymmog);
					if (logoutMsg != NULL)
					{
						processLogout(logoutMsg);
					}
					break;
				}
				case SUBSCRIBE: { // Subscribe
					receivedOverheadBytes += msg->byteLength();
					SubscribeMessage* subscribeMsg = dynamic_cast<SubscribeMessage*>(ymmog);
					if(subscribeMsg != NULL ) {
						processSubscribe(subscribeMsg);
					}
					break;
				}
				case UNSUBSCRIBE: { // Unsubscribe
					receivedOverheadBytes += msg->byteLength();
					UnSubscribeMessage* unsubscribeMsg = dynamic_cast<UnSubscribeMessage*>(ymmog);
					if ( unsubscribeMsg != NULL ) {
						processUnSubscribe(unsubscribeMsg);
					}
					break;
				}
				case PUBLISH: { // Publish
					PublishMessage* publishMsg = dynamic_cast<PublishMessage*>(ymmog);
					if( publishMsg != NULL) {
						processPublish(publishMsg);
					}
					break;
				}
				case PUBLISHEVENTLIST: {
					PublishEventListMessage* publishMsg = dynamic_cast<PublishEventListMessage*>(ymmog);
					if( publishMsg != NULL) {
						processPublishEventList(publishMsg);
					}
					break;
				}
				case CREATE: { // Create
					receivedOverheadBytes += msg->byteLength();
					CreateNodeMessage* createMsg = dynamic_cast<CreateNodeMessage*>(ymmog);
					if( createMsg != NULL ) {
						processCreate(createMsg);
					}
					break;
				}
				case REQUESTIP: {
					receivedOverheadBytes += msg->byteLength();
					RequestIpMessage *requestIpMsg = dynamic_cast<RequestIpMessage*>(ymmog);
					if (requestIpMsg != NULL ) {
						processRequestIp(requestIpMsg);
					}
					break;
				}
				case UPDATERESPIP: {
					receivedOverheadBytes += msg->byteLength();
					UpdateRespIpMessage* updateMsg = dynamic_cast<UpdateRespIpMessage*>(ymmog);
					if( updateMsg != NULL ) {
						processUpdateRespIp(updateMsg);
					}
					break;
				}
				case PING: {
					receivedOverheadBytes += msg->byteLength();
					PingMessage* pingMsg = dynamic_cast<PingMessage*>(ymmog);
					if( pingMsg != NULL ) {
						processPing(pingMsg);
					}
					break;
				}
				case PLAYERLIST: {
					receivedOverheadBytes += msg->byteLength();
					PlayerListMessage* playerListMsg = dynamic_cast<PlayerListMessage*>(ymmog);
					if( playerListMsg != NULL ) {
						processPlayerList(playerListMsg);
					}
					break;
				}
				case REQUESTNODE: {
					receivedOverheadBytes += msg->byteLength();
					RequestNodeMessage* requestMsg = dynamic_cast<RequestNodeMessage*>(ymmog);
					if( requestMsg != NULL ) {
						processRequestNode(requestMsg);
					}
					break;
				}
				case REPLACEMENT: {
					receivedOverheadBytes += msg->byteLength();
					ReplacementMessage* replaceMsg = dynamic_cast<ReplacementMessage*>(ymmog);
					if( replaceMsg != NULL ) {
						processReplacement(replaceMsg);
					}
					break;
				}
				case DELAY: {
					receivedOverheadBytes += msg->byteLength();
					DelayMessage* delayMsg = dynamic_cast<DelayMessage*>(ymmog);
					if( delayMsg != NULL ) {
						processDelay(delayMsg);
					}
					break;
				}
				case REPLACEINT: {
					receivedOverheadBytes += msg->byteLength();
					ReplaceIntermediateNodeMessage* replaceMsg = dynamic_cast<ReplaceIntermediateNodeMessage*>(ymmog);
					if ( replaceMsg != NULL ) {
						processReplaceIntermediateNode(replaceMsg);
					}
					break;
				}
				case STABILIZE: {
					StabilizeMessage* stabilizeMsg = dynamic_cast<StabilizeMessage*>(ymmog);
					if( stabilizeMsg != NULL ) {
						processStabilize(stabilizeMsg);
					}
					break;
				}
				default: {
					delete msg;
					break;
				}
			}
			overheadBytes = receivedOverheadBytes + sentOverheadBytes;
			receivedSum = receivedLogins + receivedLogouts + receivedSubscribes + receivedUnSubscribes + receivedEvents + receivedCreates + receivedRequestIps + receivedUpdateRespIps + receivedEventLists + receivedPings +receivedPlayerLists + receivedRequestNodes + receivedReplacements + receivedDelays + receivedReplaceInt;
		}
		else {
			delete msg;
		}
	}
	else if(msg->arrivedOn("from_app"))
	{
		processAppMessages(msg);
	}
}

void YMMOGBase::sendMsg(YMMOGMessage* msg, IPAddress destip) {
	msg->setSrcip(thisNode.ip.get4());
	msg->setDestip(destip);
	setLength(msg);
	msg->setTimeStamp(simTime());
	if( msg->getFailed() < 0 || msg->getFailed() > maxFailed) {
		msg->setFailed(0);
	}
	switch ( msg->getCommand() ) {
		case LOGIN: {
			sentLogins++;
			LoginMessage* loginMsg = dynamic_cast<LoginMessage*>(msg);
			if(loginMsg != NULL && loginMsg->getServiceType() == REQUEST) {
				dupedMessages.push_back( new LoginMessage(*loginMsg) );
			}
			break;
		}
		case LOGOUT: {
			sentLogouts++;
			LogoutMessage* logoutMsg = dynamic_cast<LogoutMessage*>(msg);
			if(logoutMsg != NULL && logoutMsg->getServiceType() == REQUEST) {
				dupedMessages.push_back( new LogoutMessage(*logoutMsg) );
			}
			break;
		}
		case SUBSCRIBE: {
			sentSubscribes++;
			sentOverheadBytes += msg->byteLength();
			SubscribeMessage* orginalMsg = dynamic_cast<SubscribeMessage*>(msg);
			if(orginalMsg != NULL && ( orginalMsg->getServiceType() == REQUEST || orginalMsg->getServiceType() == FORWARD) ) {
				dupedMessages.push_back( new SubscribeMessage(*orginalMsg) );
			}
			break;
		}
		case UNSUBSCRIBE: {
			sentUnSubscribes++;
			sentOverheadBytes += msg->byteLength();
			UnSubscribeMessage* orginalMsg = dynamic_cast<UnSubscribeMessage*>(msg);
			if(orginalMsg != NULL && ( orginalMsg->getServiceType() == REQUEST || orginalMsg->getServiceType() == FORWARD) ) {
				dupedMessages.push_back( new UnSubscribeMessage(*orginalMsg) );
			}
			break;
		}
		case PUBLISH: { sentEvents++; break; }
		case CREATE: {
			sentCreates++;
			sentOverheadBytes += msg->byteLength();
			CreateNodeMessage* createMsg = dynamic_cast<CreateNodeMessage*>(msg);
			if(createMsg != NULL && createMsg->getServiceType() == REQUEST ) {
				dupedMessages.push_back( new CreateNodeMessage(*createMsg) );
			}
			break;
		}
		case REQUESTIP: {
			sentRequestIps++;
			sentOverheadBytes += msg->byteLength();
			RequestIpMessage* orginalMsg = dynamic_cast<RequestIpMessage*>(msg);
			if( orginalMsg != NULL && orginalMsg->getServiceType() == REQUEST ) {
				dupedMessages.push_back( new RequestIpMessage(*orginalMsg));
			}
			break;
		}
		case UPDATERESPIP: {
			sentUpdateRespIps++;
			sentOverheadBytes += msg->byteLength();
			UpdateRespIpMessage* orginalMsg = dynamic_cast<UpdateRespIpMessage*>(msg);
			if(orginalMsg != NULL && orginalMsg->getServiceType() == REQUEST ) {
				dupedMessages.push_back( new UpdateRespIpMessage(*orginalMsg) );
			}
			break;
		}
		case PUBLISHEVENTLIST: { sentEventLists++; sentOverheadBytes += msg->byteLength(); break; }
		case PING: { sentPings++; sentOverheadBytes += msg->byteLength(); break; }
		case PLAYERLIST: {
			sentPlayerLists++;
			sentOverheadBytes += msg->byteLength();
			PlayerListMessage* orginalMsg = dynamic_cast<PlayerListMessage*>(msg);
			if(orginalMsg != NULL && ( orginalMsg->getServiceType() == REQUEST ) ) {// || orginalMsg->getServiceType() == FORWARD) ) {
				dupedMessages.push_back( new PlayerListMessage(*orginalMsg) );
			}
			break;
		}
		case REQUESTNODE: {
			sentRequestNodes++;
			sentOverheadBytes += msg->byteLength();
			RequestNodeMessage* orginalMsg = dynamic_cast<RequestNodeMessage*>(msg);
			if(orginalMsg != NULL && orginalMsg->getServiceType() == REQUEST) {
				dupedMessages.push_back( new RequestNodeMessage(*orginalMsg) );
			}
			break;
		}
		case REPLACEMENT: {
			sentReplacements++;
			sentOverheadBytes += msg->byteLength();
			ReplacementMessage* orginalMsg = dynamic_cast<ReplacementMessage*>(msg);
			if(orginalMsg != NULL && orginalMsg->getServiceType() == REQUEST) {
				dupedMessages.push_back( new ReplacementMessage(*orginalMsg) );
			}
			break;
		}
		case DELAY: { sentDelays++; sentOverheadBytes += msg->byteLength(); break; }
		case REPLACEINT: {
			sentReplaceInt++;
			sentOverheadBytes += msg->byteLength();
			ReplaceIntermediateNodeMessage* orginalMsg = dynamic_cast<ReplaceIntermediateNodeMessage*>(msg);
			if(orginalMsg != NULL && orginalMsg->getServiceType() == REQUEST) {
				dupedMessages.push_back( new ReplaceIntermediateNodeMessage(*orginalMsg) );
			}
			break;
		}
		default: { sentUndefined++; break; }
	}
	sentBytes += msg->byteLength();
	up +=msg->byteLength();
	UDPControlInfo* udpControlInfo = new UDPControlInfo();
	udpControlInfo->setDestAddr(destip);
	udpControlInfo->setDestPort(thisNode.port);
        udpControlInfo->setSrcAddr(thisNode.ip);
	udpControlInfo->setSrcPort(thisNode.port);
	msg->setControlInfo(udpControlInfo);
	send(msg,"to_udp");
	sentMessages++;
	overheadBytes = sentOverheadBytes + receivedOverheadBytes;
	messageBytes = sentBytes + receivedBytes;
	sentSum = sentLogins + sentLogouts + sentSubscribes + sentUnSubscribes + sentEvents + sentCreates + sentRequestIps + sentUpdateRespIps + sentEventLists + sentPings + sentPlayerLists +sentRequestNodes + sentReplacements + sentDelays + sentReplaceInt;
}



void YMMOGBase::setLength(YMMOGMessage* msg) {
	static const uint INT_SIZE = sizeof(int);
	static const uint SUBSPACE_SIZE = sizeof(SubSpace);
	static const uint IPADDRESS_SIZE = sizeof(IPAddress);
	static const uint BOOL_SIZE = sizeof(bool);
	static const uint DOUBLE_SIZE = sizeof(double);
	static const uint EVENT_SIZE = sizeof(Event);
	int base= 2*INT_SIZE + 2*IPADDRESS_SIZE;

	switch ( msg->getCommand() ) {
		case LOGIN: {
			LoginMessage* loginMsg = dynamic_cast<LoginMessage*>(msg);
			msg->setLength( base + 7* INT_SIZE + loginMsg->getResponsibleNodesArraySize() * SUBSPACE_SIZE);
			break;
		}
		case LOGOUT: { msg->setLength( base + INT_SIZE + BOOL_SIZE);  break; }
		case SUBSCRIBE: {
			SubscribeMessage* subscribeMsg = dynamic_cast<SubscribeMessage*>(msg);
			msg->setLength( base + INT_SIZE + IPADDRESS_SIZE + SUBSPACE_SIZE + sizeof( subscribeMsg->getGameState() ) );
			break;
		}
		case UNSUBSCRIBE: { msg->setLength( base + 2*INT_SIZE + IPADDRESS_SIZE); break; }
		case PUBLISH: { msg->setLength( base + 2*INT_SIZE + EVENT_SIZE); break; }
		case CREATE: { msg->setLength( base + 2*INT_SIZE + SUBSPACE_SIZE + BOOL_SIZE); break; }
		case REQUESTIP: { msg->setLength( base + 2*INT_SIZE + IPADDRESS_SIZE); break; }
		case UPDATERESPIP: {  msg->setLength( base + 2*INT_SIZE + IPADDRESS_SIZE); break; }
		case PUBLISHEVENTLIST: {
			PublishEventListMessage* publishMsg = dynamic_cast<PublishEventListMessage*>(msg);
			msg->setLength( base + 2*INT_SIZE + DOUBLE_SIZE + publishMsg->getEventListArraySize() * EVENT_SIZE);
			break;
		}
		case PING: { msg->setLength( base + DOUBLE_SIZE);  break; }
		case PLAYERLIST: {
			PlayerListMessage* playerListMsg = dynamic_cast<PlayerListMessage*>(msg);
			msg->setLength( base +  2*INT_SIZE + DOUBLE_SIZE + IPADDRESS_SIZE*playerListMsg->getTreeArraySize() );
			break;
		}
		case REQUESTNODE: { msg->setLength(base + 2*INT_SIZE); break; }
		case REPLACEMENT: {
			ReplacementMessage* replaceMsg = dynamic_cast<ReplacementMessage*>(msg);
			msg->setLength( base + 2*INT_SIZE + sizeof( replaceMsg->getGameState() ) );
			break;
		}
		case DELAY: { msg->setLength( base + INT_SIZE + DOUBLE_SIZE + IPADDRESS_SIZE);  break; }
		case REPLACEINT: {
			ReplaceIntermediateNodeMessage* replaceMsg = dynamic_cast<ReplaceIntermediateNodeMessage*>(msg);
			msg->setLength( base + INT_SIZE + SUBSPACE_SIZE + IPADDRESS_SIZE + replaceMsg->getIpListArraySize()*IPADDRESS_SIZE);
			break;
		}
	}
}

void YMMOGBase::deleteFromDupedMessages(YMMOGMessage* ymmog) {
	std::list<YMMOGMessage*>::iterator i = dupedMessages.begin();
	int command = ymmog->getCommand();
	while( i != dupedMessages.end() ) {
		if( (*i)->getCommand() == command) {
			switch( command ) {
				case LOGIN: {
					LoginMessage* storedMsg = dynamic_cast<LoginMessage*>(*i);
					if( storedMsg != NULL ) {
						delete *i;
						dupedMessages.erase(i);
						return;
					}
					break;
				}
				case LOGOUT: {
					LogoutMessage* receivedMsg = dynamic_cast<LogoutMessage*>(ymmog);
					LogoutMessage* storedMsg = dynamic_cast<LogoutMessage*>(*i);
					if( storedMsg != NULL && receivedMsg != NULL) {
						if( storedMsg->getId() == receivedMsg->getId() ) {
							delete *i;
							dupedMessages.erase(i);
							return;
						}
					}
					break;
				}
				case CREATE: {
					CreateNodeMessage* receivedMsg = dynamic_cast<CreateNodeMessage*>(ymmog);
					CreateNodeMessage* storedMsg = dynamic_cast<CreateNodeMessage*>(*i);
					if( storedMsg != NULL && receivedMsg != NULL) {
						if( storedMsg->getNodeType() == receivedMsg->getNodeType() && storedMsg->getJobType() == receivedMsg->getJobType() && storedMsg->getSubSpaceToCreate().getId() == receivedMsg->getSubSpaceToCreate().getId() ) {
							delete *i;
							dupedMessages.erase(i);
							return;
						}
					}
					break;
				}
				case SUBSCRIBE: {
					SubscribeMessage* receivedMsg = dynamic_cast<SubscribeMessage*>(ymmog);
					SubscribeMessage* storedMsg = dynamic_cast<SubscribeMessage*>(*i);
					if( storedMsg != NULL && receivedMsg != NULL) {
						if( ( storedMsg->getPlayerId() == receivedMsg->getPlayerId() || storedMsg->getPlayerIp() == receivedMsg->getPlayerIp() ) && storedMsg->getSubSpaceId() == receivedMsg->getSubSpaceId() ) {
							delete *i;
							dupedMessages.erase(i);
							return;
						}
					}
					break;
				}
				case UNSUBSCRIBE: {
					UnSubscribeMessage* receivedMsg = dynamic_cast<UnSubscribeMessage*>(ymmog);
					UnSubscribeMessage* storedMsg = dynamic_cast<UnSubscribeMessage*>(*i);
					if( storedMsg != NULL && receivedMsg != NULL) {
						if( (storedMsg->getPlayerId() == receivedMsg->getPlayerId() || storedMsg->getPlayerIp() == receivedMsg->getPlayerIp() ) && storedMsg->getSubSpaceId() == receivedMsg->getSubSpaceId() ) {
							delete *i;
							dupedMessages.erase(i);
							return;
						}
					}
					break;
				}
				case PUBLISH: {
					/** There should be no publishMsg's enqueued **/
					break;
				}
				case PUBLISHEVENTLIST: {
					/** There should be no PublishEventListMessage enqueued **/
					break;
				}
				case REQUESTIP: {
					RequestIpMessage* receivedMsg = dynamic_cast<RequestIpMessage*>(ymmog);
					RequestIpMessage* storedMsg = dynamic_cast<RequestIpMessage*>(*i);
					if( storedMsg != NULL && receivedMsg != NULL) {
						if( storedMsg->getSubSpaceId() == receivedMsg->getSubSpaceId() ) {
							delete *i;
							dupedMessages.erase(i);
							return;
						}
					}
					break;
				}
				case UPDATERESPIP: {
					UpdateRespIpMessage* receivedMsg = dynamic_cast<UpdateRespIpMessage*>(ymmog);
					UpdateRespIpMessage* storedMsg = dynamic_cast<UpdateRespIpMessage*>(*i);
					if( storedMsg != NULL && receivedMsg != NULL) {
						if( storedMsg->getSubSpaceId() == receivedMsg->getSubSpaceId() && storedMsg->getDirection() == receivedMsg->getDirection() && storedMsg->getDestip() == receivedMsg->getSrcip() ) {
							delete *i;
							dupedMessages.erase(i);
							return;
						}
					}
					break;
				}
				case PING: {
					/** handeld seperate **/
					break;
				}
				case PLAYERLIST: {
					PlayerListMessage* receivedMsg = dynamic_cast<PlayerListMessage*>(ymmog);
					PlayerListMessage* storedMsg = dynamic_cast<PlayerListMessage*>(*i);
					if( storedMsg != NULL && receivedMsg != NULL) {
						if( storedMsg->getSubSpaceId() == receivedMsg->getSubSpaceId() && storedMsg->getAction() == receivedMsg->getAction() ) {
							delete *i;
							dupedMessages.erase(i);
							return;
						}
					}
					break;
				}
				case REQUESTNODE: {
					RequestNodeMessage* receivedMsg = dynamic_cast<RequestNodeMessage*>(ymmog);
					RequestNodeMessage* storedMsg = dynamic_cast<RequestNodeMessage*>(*i);
					if( storedMsg != NULL && receivedMsg != NULL) {
						if( storedMsg->getSubSpaceId() == receivedMsg->getSubSpaceId() && storedMsg->getNodeType() == receivedMsg->getNodeType() ) {
							delete *i;
							dupedMessages.erase(i);
							return;
						}
					}
					break;
				}
				case REPLACEMENT: {
					ReplacementMessage* receivedMsg = dynamic_cast<ReplacementMessage*>(ymmog);
					ReplacementMessage* storedMsg = dynamic_cast<ReplacementMessage*>(*i);
					if( storedMsg != NULL && receivedMsg != NULL) {
						if( storedMsg->getSubSpaceId() == receivedMsg->getSubSpaceId() && storedMsg->getDestip() == receivedMsg->getSrcip() ) {
							delete *i;
							dupedMessages.erase(i);
							return;
						}
					}
					break;
				}
				case DELAY: {
					/** There should be no DelayMessages enqueued **/
					break;
				}
				case REPLACEINT: {
					ReplaceIntermediateNodeMessage* receivedMsg = dynamic_cast<ReplaceIntermediateNodeMessage*>(ymmog);
					ReplaceIntermediateNodeMessage* storedMsg = dynamic_cast<ReplaceIntermediateNodeMessage*>(*i);
					if( storedMsg != NULL && receivedMsg != NULL) {
						if( storedMsg->getSubSpaceId() == receivedMsg->getSubSpaceId() && storedMsg->getToReplace() == receivedMsg->getToReplace() ) {
							delete *i;
							dupedMessages.erase(i);
							return;
						}
					}
					break;
				}
				default: {
					return;
				}
			}
		}
		++i;
	}
}

void YMMOGBase::processResendTimer() {
       double min=simTime() + resendDelay;
       double now = simTime();
       std::list<YMMOGMessage*>::iterator i = dupedMessages.begin();
       while( i != dupedMessages.end() ) {
              if( (*i)->getTimeStamp() + resendDelay < now) {
                     // we have to resend it
                     (*i)->setTimeStamp(now);
                     (*i)->setFailed((*i)->getFailed() + 1);
                     if( (*i)->getFailed() > maxFailed ) {
                            delete (*i);
                            i = dupedMessages.erase(i);
                            droppedMessages++;
                     }
                     else {
                            YMMOGMessage* tmp = *i;
                            i = dupedMessages.erase(i);
                            sendMsg(tmp, tmp->getDestip());
                     }
		}
		else {
			if( ((*i)->getTimeStamp() + resendDelay < min) &&  (*i)->getTimeStamp() + resendDelay > simTime() ) min = (*i)->getTimeStamp() + resendDelay;
			++i;
		}
	}
	selfMessages++;
	cancelEvent(resendTimer);
	scheduleAt(min + 0.05, resendTimer);
}

bool YMMOGBase::findOpenPing(IPAddress destip) {
	std::list<PingMessage*>::iterator i =pings.begin();
	while( i != pings.end() ) {
		if( (*i)->getDestip() == destip ) {
			return true;
		}
		++i;
	}
	return false;
}
bool YMMOGBase::findOpenCreate( int subSpaceId, int nodeType) {
	std::list<YMMOGMessage*>::iterator i = dupedMessages.begin();
	while( i != dupedMessages.end() ) {
		if( (*i)->getCommand() == CREATE ) {
			CreateNodeMessage* tmp = dynamic_cast<CreateNodeMessage*>(*i);
			if( tmp != NULL && tmp->getSubSpaceToCreate().getId() == subSpaceId && tmp->getNodeType() ) return true;
		}
		++i;
	}
	return false;
}
bool YMMOGBase::findOpenSubScribe(int subSpaceId, IPAddress destip) {
	std::list<YMMOGMessage*>::iterator i=dupedMessages.begin();
	while( i != dupedMessages.end() ) {
		if( (*i)->getCommand() == SUBSCRIBE ) {
			SubscribeMessage* subscribeMsg = dynamic_cast<SubscribeMessage*>(*i);
			if( subscribeMsg != NULL && subscribeMsg->getSubSpaceId() == subSpaceId && subscribeMsg->getDestip() == destip)
				return true;
		}
		++i;
	}
	return false;
}
bool YMMOGBase::findOpenLogin(IPAddress playerip ) {
	std::list<YMMOGMessage*>::iterator i= notReadyMessages.begin();
	while( i != notReadyMessages.end() ) {
		if( (*i)->getCommand() == LOGIN) {
			LoginMessage* loginMsg = dynamic_cast<LoginMessage*>(*i);
			if( loginMsg != NULL && loginMsg->getSrcip() == playerip)
				return true;
		}
		++i;
	}
	i= dupedMessages.begin();
	while( i != dupedMessages.end() ) {
		if( (*i)->getCommand() == LOGIN) {
			LoginMessage* loginMsg = dynamic_cast<LoginMessage*>(*i);
			if( loginMsg != NULL && loginMsg->getSrcip() == playerip)
				return true;
		}
		++i;
	}
	return false;
}
bool YMMOGBase::findOpenUnSubscribe(int subSpaceId, IPAddress destip) {
	std::list<YMMOGMessage*>::iterator i=dupedMessages.begin();
	while( i != dupedMessages.end() ) {
		if( (*i)->getCommand() == UNSUBSCRIBE ) {
			UnSubscribeMessage* unSubscribeMsg = dynamic_cast<UnSubscribeMessage*>(*i);
			if( unSubscribeMsg != NULL && unSubscribeMsg->getSubSpaceId() == subSpaceId && unSubscribeMsg->getDestip() == destip)
				return true;
		}
		++i;
	}
	return false;
}
bool YMMOGBase::findOpenRequestNode(int subSpaceId, int nodeType) {
	std::list<YMMOGMessage*>::iterator i= dupedMessages.begin();
	while( i != dupedMessages.end() ) {
		if( (*i)->getCommand() == REQUESTNODE ) {
			RequestNodeMessage* requestMsg = dynamic_cast<RequestNodeMessage*>(*i);
			if( requestMsg != NULL && requestMsg->getSubSpaceId() == subSpaceId && requestMsg->getNodeType() == nodeType)
				return true;
		}
		++i;
	}
	return false;
}

void YMMOGBase::resetPing(IPAddress destip) {
	std::list<PingMessage*>::iterator i= pings.begin();
	while( i != pings.end() ) {
		if( (*i)->getDestip() == destip ) {
			(*i)->setTimeStamp(simTime());
			(*i)->setFailed(0);
			break;
		}
		++i;
	}
	i =  playerPings.begin();
	while( i != playerPings.end() ) {
		if( (*i)->getDestip() == destip ) {
			delete *i;
			i = playerPings.erase(i);
		}
		else { ++i; }
	}
	return;
}

void YMMOGBase::deleteFromOpenPings(IPAddress destip) {
	std::list<PingMessage*>::iterator i= pings.begin();
	while( i != pings.end() ) {
		if( (*i)->getDestip() == destip ) {
			delete *i;
			i = pings.erase(i);
			return;
		}
		++i;
	}
}
void YMMOGBase::sendPing( PingMessage* openPing) {
	ev << "Warning this is a Dummy Function! sendPing";
}
void YMMOGBase::processSelfTimers(cMessage *msg) {
	ev << "Warning this is a Dummy Function! processSelfTimers";
}
void YMMOGBase::processAppMessages(cMessage *msg) {
	ev << "Warning this is a Dummy Function! processAppMessages";
}
void YMMOGBase::processBandwithTimer() {
	cancelEvent(bandwithTimer);
	upstream->record( up / bandwithDelay );
	downstream->record( down / bandwithDelay );
	up = 0;
	down = 0;
	scheduleAt(simTime() + bandwithDelay, bandwithTimer);
}
void YMMOGBase::processStabilizeTimer() {
	ev << "Warning this is a Dummy Function! processStabilizeTimer()";
}
void YMMOGBase::processCreate(CreateNodeMessage* createMsg) {
	// No default handling
	ev << "Warning this is a Dummy Function! processCreate";
}
void YMMOGBase::processDelay(DelayMessage* delayMsg) {
	ev << "Warning this is a Dummy Function! processDelay(DelayMessage* delayMsg)";
}
void YMMOGBase::processLogin(LoginMessage* loginMsg) {
	ev << "Warning this is a Dummy Function! processLogin(LoginMessage* loginMsg)";
}
void YMMOGBase::processLogout(LogoutMessage* logoutMsg) {
	ev << "Warning this is a Dummy Function! processLogout(LogoutMessage* logoutMsg)";
}
void YMMOGBase::processPing(PingMessage* pingMsg) {
	ev << "Warning this is a Dummy Function! processPing(PingMessage* pingMsg)";
}
void YMMOGBase::processPlayerList(PlayerListMessage* playerListMsg) {
	ev << "Warning this is a Dummy Function! processPlayerList(PlayerListMessage* playerListMsg)";
}
void YMMOGBase::processPublish(PublishMessage* publishMsg) {
	ev << "Warning this is a Dummy Function! processPublish(PublishMessage* publishMsg";
}
void YMMOGBase::processPublishEventList(PublishEventListMessage* publishMsg) {
	ev << "Warning this is a Dummy Function! processPublishEventList(PublishEventListMessage* publishMsg)";
}
void YMMOGBase::processReplaceIntermediateNode(ReplaceIntermediateNodeMessage* replaceMsg) {
	ev << "Warning this is a Dummy Function! processReplaceIntermediateNode(ReplaceIntermediateNodeMessage* replaceMsg)";
}
void YMMOGBase::processReplacement(ReplacementMessage* replaceMsg) {
	ev << "Warning this is a Dummy Function! processReplacement(ReplacementMessage* replaceMsg)";
}
void YMMOGBase::processRequestIp(RequestIpMessage* requestMsg) {
	ev << "Warning this is a Dummy Function! processRequestIp(RequestIpMessage* requestMsg)";
}
void YMMOGBase::processRequestNode(RequestNodeMessage* requestMsg) {
	ev << "Warning this is a Dummy Function! processRequestNode(RequestNodeMessage* requestMsg)";
}
void YMMOGBase::processStabilize(StabilizeMessage* smsg){
	ev << "Warning this is a Dummy Function! processStabilize(StabilizeMessage* smsg)";
}
void YMMOGBase::processSubscribe(SubscribeMessage* subscribeMsg) {
	ev << "Warning this is a Dummy Function! processSubscribe(SubscribeMessage* subscribeMsg)";
}
void YMMOGBase::processUnSubscribe(UnSubscribeMessage* unsubscribeMsg) {
	ev << "Warning this is a Dummy Function! processUnSubscribe(UnSubscribeMessage* unsubscribeMsg)";
}
void YMMOGBase::processUpdateRespIp(UpdateRespIpMessage* updateMsg) {
	ev << "Warning this is a Dummy Function! processUpdateRespIp(UpdateRespIpMessage* updateMsg)";
}
