#ifndef YMMOGBASE_H_
#define YMMOGBASE_H_

#include "YMMOGMessage_m.h"
#include "pendingMessages.h"
#include <BootstrapOracle.h>
#include <BootstrapOracleAccess.h>
#include <UDPAppBase.h>
#include <omnetpp.h>
#include <NodeHandle.h>
#include <IPvXAddress.h>
#include <CommonMessages_m.h>
#include <UDPControlInfo_m.h>
#include <UDPSocket.h>
#include <IPAddressResolver.h>
#include <InterfaceTable.h>
#include <IPv4InterfaceData.h>
#include <GlobalStatistics.h>
#include <GlobalStatisticsAccess.h>

class YMMOGBase :public cSimpleModule
{
public:
		virtual int numInitStages() const {return 6;}
		// OMNeT++
		void initialize(int stage);
		virtual void initialize();
		virtual void finish();
		void handleMessage(cMessage* msg);
		
		void sendMsg(YMMOGMessage* msg, IPAddress destip);
		void setLength(YMMOGMessage* msg);
		
		
		std::list<YMMOGMessage*> notReadyMessages;
		std::list<YMMOGMessage*> dupedMessages;
		void deleteFromDupedMessages(YMMOGMessage* ymmog);
		

		bool findOpenSubScribe( int subSpaceId, IPAddress destip);
		bool findOpenUnSubscribe( int subSpaceId, IPAddress destip);
		bool findOpenPing(IPAddress destip);
		bool findOpenPlayerList(int subSpaceId);
		bool findOpenRequestNode(int subSpaceId, int nodeType);
		bool findOpenCreate( int subSpaceId, int nodeType);
		bool findOpenLogin(IPAddress playerip );
		
		virtual void initializeStats();
		virtual void finishStats();
		
		virtual void processSelfTimers(cMessage* msg);
		virtual void processAppMessages(cMessage* msg);		
		
		virtual void processResendTimer();
		virtual void processBandwithTimer();
		virtual void processStabilizeTimer();
		
		// Message Processing functions
		virtual void processLogin(LoginMessage* loginMsg);		
		virtual void processLogout(LogoutMessage* logoutMsg);
		virtual void processCreate(CreateNodeMessage* createMsg);
		virtual void processSubscribe(SubscribeMessage* subscribeMsg);
		virtual void processUnSubscribe(UnSubscribeMessage* unsubscribeMsg);
		virtual void processPublish(PublishMessage* publishMsg);
		virtual void processPublishEventList(PublishEventListMessage* publishMsg);
		virtual void processRequestIp(RequestIpMessage* requestMsg);
		virtual void processDelay(DelayMessage* delayMsg);
		virtual void processReplaceIntermediateNode(ReplaceIntermediateNodeMessage* replaceMsg);
		virtual void processPlayerList(PlayerListMessage* playerListMsg);
		virtual void processRequestNode(RequestNodeMessage* requestMsg);
		virtual void processUpdateRespIp(UpdateRespIpMessage* updateMsg);		
		virtual void processReplacement(ReplacementMessage* replaceMsg);
		virtual void processPing(PingMessage* pingMsg);
		virtual void processStabilize(StabilizeMessage* smsg);



		// Ping
		std::list<PingMessage*> pings;
		std::list<PingMessage*> playerPings;
		virtual void sendPing( PingMessage* openPing);
		void deleteFromOpenPings(IPAddress destip);
		void resetPing(IPAddress destip);
		//stabilize
		double stabilizeDelay;
		double bandwithDelay;		
		double resendDelay;
		double pingDelay;
		
		cMessage* resendTimer;
		cMessage* pingTimer;
		cMessage* stabilizeTimer;
		cMessage* bandwithTimer;

		int selfMessages;
		int maxFailed;
		int droppedMessages;


		
		bool debug;
		int localPort;
		double sessionBegin;

		// Message Log
		int sentMessages;
		int receivedMessages;
		
		cOutVector* upstream;
		cOutVector* downstream;
		unsigned int up;
		unsigned int down;
		// Summary
		int sentSum;
		int receivedSum;
		unsigned int overheadBytes;
		unsigned int messageBytes;
		unsigned int sentOverheadBytes;
		unsigned int receivedOverheadBytes;
		unsigned int receivedBytes;
		unsigned int sentBytes;
		
		// Sent Variables:
		int sentLogins;
		int sentLogouts;
		int sentSubscribes;
		int sentUnSubscribes;
		int sentEvents;
		int sentCreates;
		int sentRequestIps;
		int sentUpdateRespIps;
		int sentEventLists;
		int sentPings;
		int sentPlayerLists;
		int sentRequestNodes;
		int sentReplacements;
		int sentDelays;
		int sentReplaceInt;
		int sentUndefined;
		// Received Variables:
		int receivedLogins;
		int receivedLogouts;	
		int receivedSubscribes;
		int receivedUnSubscribes;
		int receivedEvents;
		int receivedCreates;
		int receivedRequestIps;
		int receivedUpdateRespIps;
		int receivedEventLists;
		int receivedPings;
		int receivedPlayerLists;
		int receivedRequestNodes;
		int receivedReplacements;
		int receivedDelays;
		int receivedReplaceInt;
		
		//module references
		BootstrapOracle* bootstrapOracle;
		NodeHandle thisNode;
		GlobalStatistics* globalStatistics;         //*< pointer to GlobalStatistics module in this node */
};

#endif /*YMMOGBASE_H_*/
