//
// Copyright (C) 2007 Institut fuer Telematik, Universitaet Karlsruhe (TH)
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//

/**
 * @file P2pns.h
 * @author Ingmar Baumgart
 */

#ifndef __P2PNS_H_
#define __P2PNS_H_

#include <omnetpp.h>

#include <OverlayKey.h>
#include <SHA1.h>
#include <CommonMessages_m.h>

#include <BaseApp.h>
#include <RpcMacros.h>

#include "P2pnsCache.h"

/**
 * Implementation of P2PNS: A distributed name service for P2PSIP
 *
 * Implementation of P2PNS: A distributed name service for P2PSIP
 */
class P2pns : public BaseApp
{
public:
    P2pns();
    virtual ~P2pns();

private:
    void initializeApp(int stage);
    void finishApp();
    void handleTimerEvent(cMessage* msg);

    bool handleRpc(BaseCallMessage* msg);
    void handleRpcResponse(BaseResponseMessage* msg, int rpcId, simtime_t rtt);
    void handleRpcTimeout(BaseCallMessage* msg, const TransportAddress& dest,
                          int rpcId, const OverlayKey& destKey);

    void p2pnsRegisterRpc(P2pnsRegisterCall* registerCall);
    void p2pnsResolveRpc(P2pnsResolveCall* registerCall);

    void handleDHTputCAPIResponse(DHTputCAPIResponse* putResponse);
    void handleDHTgetCAPIResponse(DHTgetCAPIResponse* gettResponse);
    void handleLookupResponse(LookupResponse* lookupResponse);

    std::map<unsigned int, BaseCallMessage*> rpcIdMap; /**< List of the Rpc Ids of the messages sent following the reception of an rpc request (the second member) */
    P2pnsCache *p2pnsCache; /**< pointer to the name cache module */
    bool twoStageResolution; /**< Use the two stage name resolution (KBR/DHt) */
    
};

#endif
