#ifndef _XMLRPCINTERFACE_H__
#define _XMLRPCINTERFACE_H__

#include <omnetpp.h>
#include <CommonMessages_m.h>
#include <DHTMessage_m.h>
#include <BaseOverlay.h>
#include <BaseApp.h>
#include <tunoutscheduler.h>
#include <realtimescheduler.h>
#include <XmlRpc.h>

const int XMLRPC_TIMEOUT=30;


/**
 * Applicaton that communicates with a realword application via a socket
 */
class XmlRpcInterface : public BaseApp, public XmlRpc::XmlRpcServer
{
protected:
    unsigned int mtu;
//    int gateIndexNetwOut;

    cMessage* timeout_msg;

    cMessage* packetNotification; // used by TunOutScheduler to notify about new packets
    RealtimeScheduler::PacketBuffer packetBuffer; // received packets are stored here
    RealtimeScheduler* scheduler;

    //! Reads the http header
    bool readHeader(char *buf, uint32_t length);

    //! Reads the request (based on the content-length header value)
    bool readRequest(char *buf, uint32_t length);

    //! Executes the request and writes the resulting response
    bool writeResponse();

    //! Possible IO states for the connection
    enum ServerConnectionState { READ_HEADER, READ_REQUEST, EXECUTE_REQUEST,
			     WRITE_RESPONSE };

    //! Current IO state for the connection
    ServerConnectionState _connectionState;


    //! Request headers
    std::string _header;

    //! Number of bytes expected in the request body (parsed from header)
    int _contentLength;

    //! Request body
    std::string _request;

    //! Response
    std::string _response;

    //! Number of bytes of the response written so far
    int _bytesWritten;

    //! Whether to keep the current client connection open for further requests
    bool _keepAlive;
    
    //! the fd for the current app socket
    int appFd;

    XmlRpc::XmlRpcServerMethod* _localLookup;
    XmlRpc::XmlRpcServerMethod* _lookup;
    XmlRpc::XmlRpcServerMethod* _register;
    XmlRpc::XmlRpcServerMethod* _resolve;
    XmlRpc::XmlRpcServerMethod* _put;
    XmlRpc::XmlRpcServerMethod* _get;
    XmlRpc::XmlRpcServerMethod* _joinOverlay;
    
    void handleRealworldPacket(char *buf, uint32_t len);
    void handleCommonAPIPacket(cMessage *msg);
    void handleRpcResponse(BaseResponseMessage* msg, int rpcId,
                              simtime_t rtt);


public:
//    Module_Class_Members(XmlRpcInterface, cModule, 0);


    virtual void initializeApp(int stage);

    /**
     * The "main loop". Every message that is received or send is handled
     * by this method
     */
    virtual void handleMessage(cMessage *msg);

    void localLookup(XmlRpc::XmlRpcValue& params, XmlRpc::XmlRpcValue& result);
    void lookup(XmlRpc::XmlRpcValue& params, XmlRpc::XmlRpcValue& result);
    void p2pnsRegister(XmlRpc::XmlRpcValue& params, XmlRpc::XmlRpcValue& result);
    void p2pnsResolve(XmlRpc::XmlRpcValue& params, XmlRpc::XmlRpcValue& result);
    void put(XmlRpc::XmlRpcValue& params, XmlRpc::XmlRpcValue& result);
    void get(XmlRpc::XmlRpcValue& params, XmlRpc::XmlRpcValue& result);
    void joinOverlay(XmlRpc::XmlRpcValue& params, XmlRpc::XmlRpcValue& result);

};

#endif
